@@ -1,22 +1,23 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000-2004
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2000-2010 Oracle.  All rights reserved.
  *
- * $Id: DataCursor.java,v 1.48 2004/06/04 14:48:20 mark Exp $
  */
 
 package com.sleepycat.collections;
 
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.je.Cursor;
+import com.sleepycat.je.CursorConfig;
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.JoinConfig;
 import com.sleepycat.je.JoinCursor;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
+import com.sleepycat.util.keyrange.KeyRange;
+import com.sleepycat.util.keyrange.RangeCursor;
 
 /**
  * Represents a Berkeley DB cursor and adds support for indices, bindings and
@@ -29,12 +30,19 @@ import com.sleepycat.je.OperationStatus;
  */
 final class DataCursor implements Cloneable {
 
+    /** Repositioned exactly to the key/data pair given. */
+    static final int REPOS_EXACT = 0;
+    /** Repositioned on a record following the key/data pair given. */
+    static final int REPOS_NEXT = 1;
+    /** Repositioned failed, no records on or after the key/data pair given. */
+    static final int REPOS_EOF = 2;
+
     private RangeCursor cursor;
     private JoinCursor joinCursor;
     private DataView view;
     private KeyRange range;
     private boolean writeAllowed;
-    private boolean dirtyRead;
+    private boolean readUncommitted;
     private DatabaseEntry keyThang;
     private DatabaseEntry valueThang;
     private DatabaseEntry primaryKeyThang;
@@ -47,28 +55,40 @@ final class DataCursor implements Cloneable {
     DataCursor(DataView view, boolean writeAllowed)
         throws DatabaseException {
 
-        init(view, writeAllowed, null);
+        init(view, writeAllowed, null, null);
+    }
+
+    /**
+     * Creates a cursor for a given view.
+     */
+    DataCursor(DataView view, boolean writeAllowed, CursorConfig config)
+        throws DatabaseException {
+
+        init(view, writeAllowed, config, null);
     }
 
     /**
      * Creates a cursor for a given view and single key range.
+     * Used by unit tests.
      */
     DataCursor(DataView view, boolean writeAllowed, Object singleKey)
         throws DatabaseException {
 
-        init(view, writeAllowed, view.subRange(singleKey));
+        init(view, writeAllowed, null, view.subRange(view.range, singleKey));
     }
 
     /**
      * Creates a cursor for a given view and key range.
+     * Used by unit tests.
      */
     DataCursor(DataView view, boolean writeAllowed,
                Object beginKey, boolean beginInclusive,
                Object endKey, boolean endInclusive)
         throws DatabaseException {
 
-        init(view, writeAllowed,
-             view.subRange(beginKey, beginInclusive, endKey, endInclusive));
+        init(view, writeAllowed, null,
+             view.subRange
+                (view.range, beginKey, beginInclusive, endKey, endInclusive));
     }
 
     /**
@@ -88,7 +108,7 @@ final class DataCursor implements Cloneable {
             cursors[i] = indexCursors[i].cursor.getCursor();
         }
         joinCursor = view.db.join(cursors, joinConfig);
-        init(view, false, null);
+        init(view, false, null, null);
         if (closeIndexCursors) {
             indexCursorsToClose = indexCursors;
         }
@@ -130,33 +150,54 @@ final class DataCursor implements Cloneable {
     /**
      * Constructor helper.
      */
-    private void init(DataView view, boolean writeAllowed, KeyRange range)
+    private void init(DataView view,
+                      boolean writeAllowed,
+                      CursorConfig config,
+                      KeyRange range)
         throws DatabaseException {
 
+        if (config == null) {
+            config = view.cursorConfig;
+        }
         this.view = view;
         this.writeAllowed = writeAllowed && view.writeAllowed;
         this.range = (range != null) ? range : view.range;
-        dirtyRead = view.dirtyReadEnabled;
-
+        readUncommitted = config.getReadUncommitted() ||
+                          view.currentTxn.isReadUncommitted();
         initThangs();
 
         if (joinCursor == null) {
-            cursor = new RangeCursor(view, this.range, this.writeAllowed);
+            cursor = new MyRangeCursor
+                (this.range, config, view, this.writeAllowed);
         }
     }
 
     /**
      * Constructor helper.
      */
-    private void initThangs()
-        throws DatabaseException {
-
+    private void initThangs() {
         keyThang = new DatabaseEntry();
         primaryKeyThang = view.isSecondary() ? (new DatabaseEntry())
                                              : keyThang;
         valueThang = new DatabaseEntry();
     }
 
+    /**
+     * Set entries from given byte arrays.
+     */
+    private void setThangs(byte[] keyBytes,
+                           byte[] priKeyBytes,
+                           byte[] valueBytes) {
+
+        keyThang.setData(KeyRange.copyBytes(keyBytes));
+
+        if (keyThang != primaryKeyThang) {
+            primaryKeyThang.setData(KeyRange.copyBytes(priKeyBytes));
+        }
+
+        valueThang.setData(KeyRange.copyBytes(valueBytes));
+    }
+
     /**
      * Closes the associated cursor.
      */
@@ -171,7 +212,7 @@ final class DataCursor implements Cloneable {
         if (cursor != null) {
             Cursor toClose = cursor.getCursor();
             cursor = null;
-            view.currentTxn.closeCursor(toClose );
+            view.currentTxn.closeCursor(toClose);
         }
         if (indexCursorsToClose != null) {
             DataCursor[] toClose = indexCursorsToClose;
@@ -182,6 +223,93 @@ final class DataCursor implements Cloneable {
         }
     }
 
+    /**
+     * Repositions to a given raw key/data pair, or just past it if that record
+     * has been deleted.
+     *
+     * @return REPOS_EXACT, REPOS_NEXT or REPOS_EOF.
+     */
+    int repositionRange(byte[] keyBytes,
+                        byte[] priKeyBytes,
+                        byte[] valueBytes,
+                        boolean lockForWrite)
+        throws DatabaseException {
+
+        LockMode lockMode = getLockMode(lockForWrite);
+        OperationStatus status = null;
+
+        /* Use the given key/data byte arrays. */
+        setThangs(keyBytes, priKeyBytes, valueBytes);
+
+        /* Position on or after the given key/data pair. */
+        if (view.dupsAllowed) {
+            status = cursor.getSearchBothRange(keyThang, primaryKeyThang,
+                                               valueThang, lockMode);
+        }
+        if (status != OperationStatus.SUCCESS) {
+            status = cursor.getSearchKeyRange(keyThang, primaryKeyThang,
+                                              valueThang, lockMode);
+        }
+
+        /* Return the result of the operation. */
+        if (status == OperationStatus.SUCCESS) {
+            if (!KeyRange.equalBytes(keyBytes, 0, keyBytes.length,
+                                     keyThang.getData(),
+                                     keyThang.getOffset(),
+                                     keyThang.getSize())) {
+                return REPOS_NEXT;
+            }
+            if (view.dupsAllowed) {
+                DatabaseEntry thang = view.isSecondary() ? primaryKeyThang
+                                                         : valueThang;
+                byte[] bytes = view.isSecondary() ? priKeyBytes
+                                                  : valueBytes;
+                if (!KeyRange.equalBytes(bytes, 0, bytes.length,
+                                         thang.getData(),
+                                         thang.getOffset(),
+                                         thang.getSize())) {
+                    return REPOS_NEXT;
+                }
+            }
+            return REPOS_EXACT;
+        } else {
+            return REPOS_EOF;
+        }
+    }
+
+    /**
+     * Repositions to a given raw key/data pair.
+     *
+     * @throws IllegalStateException when the database has unordered keys or
+     * unordered duplicates.
+     *
+     * @return whether the search succeeded.
+     */
+    boolean repositionExact(byte[] keyBytes,
+                            byte[] priKeyBytes,
+                            byte[] valueBytes,
+                            boolean lockForWrite)
+        throws DatabaseException {
+
+        LockMode lockMode = getLockMode(lockForWrite);
+        OperationStatus status = null;
+
+        /* Use the given key/data byte arrays. */
+        setThangs(keyBytes, priKeyBytes, valueBytes);
+
+        /* Position on the given key/data pair. */
+        if (view.recNumRenumber) {
+            /* getSearchBoth doesn't work with recno-renumber databases. */
+            status = cursor.getSearchKey(keyThang, primaryKeyThang,
+                                         valueThang, lockMode);
+        } else {
+            status = cursor.getSearchBoth(keyThang, primaryKeyThang,
+                                          valueThang, lockMode);
+        }
+
+        return (status == OperationStatus.SUCCESS);
+    }
+
     /**
      * Returns the view for this cursor.
      */
@@ -210,24 +338,39 @@ final class DataCursor implements Cloneable {
     /**
      * Returns the key object for the last record read.
      */
-    Object getCurrentKey()
-        throws DatabaseException {
-
-        if (view.keyBinding == null) {
-            throw new UnsupportedOperationException();
-        }
-        return view.makeKey(keyThang);
+    Object getCurrentKey() {
+        return view.makeKey(keyThang, primaryKeyThang);
     }
 
     /**
      * Returns the value object for the last record read.
      */
-    Object getCurrentValue()
-        throws DatabaseException {
-
+    Object getCurrentValue() {
         return view.makeValue(primaryKeyThang, valueThang);
     }
 
+    /**
+     * Returns the internal key entry.
+     */
+    DatabaseEntry getKeyThang() {
+        return keyThang;
+    }
+
+    /**
+     * Returns the internal primary key entry, which is the same object as the
+     * key entry if the cursor is not for a secondary database.
+     */
+    DatabaseEntry getPrimaryKeyThang() {
+        return primaryKeyThang;
+    }
+
+    /**
+     * Returns the internal value entry.
+     */
+    DatabaseEntry getValueThang() {
+        return valueThang;
+    }
+
     /**
      * Returns whether record number access is allowed.
      */
@@ -255,7 +398,7 @@ final class DataCursor implements Cloneable {
             return DbCompat.getRecordNumber(keyThang);
         }
     }
-    
+
     /**
      * Binding version of Cursor.getCurrent(), no join cursor allowed.
      */
@@ -266,7 +409,7 @@ final class DataCursor implements Cloneable {
         return cursor.getCurrent(keyThang, primaryKeyThang, valueThang,
                                  getLockMode(lockForWrite));
     }
-    
+
     /**
      * Binding version of Cursor.getFirst(), join cursor is allowed.
      */
@@ -281,7 +424,7 @@ final class DataCursor implements Cloneable {
                                    lockMode);
         }
     }
-    
+
     /**
      * Binding version of Cursor.getNext(), join cursor is allowed.
      */
@@ -296,7 +439,7 @@ final class DataCursor implements Cloneable {
                                   lockMode);
         }
     }
-    
+
     /**
      * Binding version of Cursor.getNext(), join cursor is allowed.
      */
@@ -306,12 +449,15 @@ final class DataCursor implements Cloneable {
         LockMode lockMode = getLockMode(lockForWrite);
         if (joinCursor != null) {
             return joinCursor.getNext(keyThang, valueThang, lockMode);
+        } else if (view.dupsView) {
+            return cursor.getNext
+                (keyThang, primaryKeyThang, valueThang, lockMode);
         } else {
-            return cursor.getNextNoDup(keyThang, primaryKeyThang, valueThang,
-                                       lockMode);
+            return cursor.getNextNoDup
+                (keyThang, primaryKeyThang, valueThang, lockMode);
         }
     }
-    
+
     /**
      * Binding version of Cursor.getNextDup(), no join cursor allowed.
      */
@@ -319,10 +465,15 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getNextDup(keyThang, primaryKeyThang, valueThang,
-                                 getLockMode(lockForWrite));
+        if (view.dupsView) {
+            return null;
+        } else {
+            return cursor.getNextDup
+                (keyThang, primaryKeyThang, valueThang,
+                 getLockMode(lockForWrite));
+        }
     }
-    
+
     /**
      * Binding version of Cursor.getLast(), no join cursor allowed.
      */
@@ -333,7 +484,7 @@ final class DataCursor implements Cloneable {
         return cursor.getLast(keyThang, primaryKeyThang, valueThang,
                               getLockMode(lockForWrite));
     }
-    
+
     /**
      * Binding version of Cursor.getPrev(), no join cursor allowed.
      */
@@ -344,7 +495,7 @@ final class DataCursor implements Cloneable {
         return cursor.getPrev(keyThang, primaryKeyThang, valueThang,
                               getLockMode(lockForWrite));
     }
-    
+
     /**
      * Binding version of Cursor.getPrevNoDup(), no join cursor allowed.
      */
@@ -352,10 +503,18 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getPrevNoDup(keyThang, primaryKeyThang, valueThang,
-                                   getLockMode(lockForWrite));
+        LockMode lockMode = getLockMode(lockForWrite);
+        if (view.dupsView) {
+            return null;
+        } else if (view.dupsView) {
+            return cursor.getPrev
+                (keyThang, primaryKeyThang, valueThang, lockMode);
+        } else {
+            return cursor.getPrevNoDup
+                (keyThang, primaryKeyThang, valueThang, lockMode);
+        }
     }
-    
+
     /**
      * Binding version of Cursor.getPrevDup(), no join cursor allowed.
      */
@@ -363,10 +522,15 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.getPrevDup(keyThang, primaryKeyThang, valueThang,
-                                 getLockMode(lockForWrite));
+        if (view.dupsView) {
+            return null;
+        } else {
+            return cursor.getPrevDup
+                (keyThang, primaryKeyThang, valueThang,
+                 getLockMode(lockForWrite));
+        }
     }
-    
+
     /**
      * Binding version of Cursor.getSearchKey(), no join cursor allowed.
      * Searches by record number in a BTREE-RECNO db with RECNO access.
@@ -376,13 +540,21 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        if (view.useKey(key, value, keyThang, range)) {
-            return doGetSearchKey(lockForWrite);
+        if (view.dupsView) {
+            if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
+                KeyRange.copy(view.dupsKey, keyThang);
+                return cursor.getSearchBoth
+                    (keyThang, primaryKeyThang, valueThang,
+                     getLockMode(lockForWrite));
+            }
         } else {
-            return OperationStatus.NOTFOUND;
+            if (view.useKey(key, value, keyThang, range)) {
+                return doGetSearchKey(lockForWrite);
+            }
         }
+        return OperationStatus.NOTFOUND;
     }
-    
+
     /**
      * Pass-thru version of Cursor.getSearchKey().
      * Searches by record number in a BTREE-RECNO db with RECNO access.
@@ -399,7 +571,7 @@ final class DataCursor implements Cloneable {
                                        valueThang, lockMode);
         }
     }
-    
+
     /**
      * Binding version of Cursor.getSearchKeyRange(), no join cursor allowed.
      */
@@ -408,28 +580,46 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        if (view.useKey(key, value, keyThang, range)) {
-            return cursor.getSearchKeyRange(keyThang, primaryKeyThang,
-                                            valueThang,
-                                            getLockMode(lockForWrite));
+        LockMode lockMode = getLockMode(lockForWrite);
+        if (view.dupsView) {
+            if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
+                KeyRange.copy(view.dupsKey, keyThang);
+                return cursor.getSearchBothRange
+                    (keyThang, primaryKeyThang, valueThang, lockMode);
+            }
         } else {
-            return OperationStatus.NOTFOUND;
+            if (view.useKey(key, value, keyThang, range)) {
+                return cursor.getSearchKeyRange
+                    (keyThang, primaryKeyThang, valueThang, lockMode);
+            }
         }
+        return OperationStatus.NOTFOUND;
     }
-    
+
     /**
-     * Binding version of Cursor.getSearchBoth(), no join cursor allowed.
-     * Unlike SecondaryCursor.getSearchBoth, for a secondary this searches for
-     * the primary value not the primary key.
+     * Find the given key and value using getSearchBoth if possible or a
+     * sequential scan otherwise, no join cursor allowed.
      */
-    OperationStatus getSearchBoth(Object key, Object value,
-                                  boolean lockForWrite)
+    OperationStatus findBoth(Object key, Object value, boolean lockForWrite)
         throws DatabaseException {
 
         checkNoJoinCursor();
         LockMode lockMode = getLockMode(lockForWrite);
         view.useValue(value, valueThang, null);
-        if (view.useKey(key, value, keyThang, range)) {
+        if (view.dupsView) {
+            if (view.useKey(key, value, primaryKeyThang, view.dupsRange)) {
+                KeyRange.copy(view.dupsKey, keyThang);
+                if (otherThang == null) {
+                    otherThang = new DatabaseEntry();
+                }
+                OperationStatus status = cursor.getSearchBoth
+                    (keyThang, primaryKeyThang, otherThang, lockMode);
+                if (status == OperationStatus.SUCCESS &&
+                    KeyRange.equalBytes(otherThang, valueThang)) {
+                    return status;
+                }
+            }
+        } else if (view.useKey(key, value, keyThang, range)) {
             if (view.isSecondary()) {
                 if (otherThang == null) {
                     otherThang = new DatabaseEntry();
@@ -440,35 +630,32 @@ final class DataCursor implements Cloneable {
                                                              lockMode);
                 while (status == OperationStatus.SUCCESS) {
                     if (KeyRange.equalBytes(otherThang, valueThang)) {
-                        break;
+                        return status;
                     }
                     status = cursor.getNextDup(keyThang, primaryKeyThang,
                                                otherThang, lockMode);
                 }
                 /* if status != SUCCESS set range cursor to invalid? */
-                return status;
             } else {
                 return cursor.getSearchBoth(keyThang, null, valueThang,
                                             lockMode);
             }
-        } else {
-            return OperationStatus.NOTFOUND;
         }
+        return OperationStatus.NOTFOUND;
     }
 
     /**
      * Find the given value using getSearchBoth if possible or a sequential
      * scan otherwise, no join cursor allowed.
      */
-    OperationStatus find(Object value, boolean findFirst)
+    OperationStatus findValue(Object value, boolean findFirst)
         throws DatabaseException {
 
         checkNoJoinCursor();
-        LockMode lockMode = getLockMode(false);
 
         if (view.entityBinding != null && !view.isSecondary() &&
             (findFirst || !view.dupsAllowed)) {
-            return getSearchBoth(null, value, false);
+            return findBoth(null, value, false);
         } else {
             if (otherThang == null) {
                 otherThang = new DatabaseEntry();
@@ -493,7 +680,11 @@ final class DataCursor implements Cloneable {
         throws DatabaseException {
 
         checkNoJoinCursor();
-        return cursor.count();
+        if (view.dupsView) {
+            return 1;
+        } else {
+            return cursor.count();
+        }
     }
 
     /**
@@ -504,7 +695,7 @@ final class DataCursor implements Cloneable {
 
         checkWriteAllowed(false);
         view.useValue(value, valueThang, keyThang);
-        
+
         /*
          * Workaround for a DB core problem: With HASH type a put() with
          * different data is allowed.
@@ -535,7 +726,7 @@ final class DataCursor implements Cloneable {
 
         checkWriteAllowed(false);
         view.useValue(value, valueThang, null); /* why no key check? */
-        return cursor.putAfter(valueThang);
+        return cursor.putAfter(keyThang, valueThang);
     }
 
     /**
@@ -546,7 +737,7 @@ final class DataCursor implements Cloneable {
 
         checkWriteAllowed(false);
         view.useValue(value, valueThang, keyThang);
-        return cursor.putBefore(valueThang);
+        return cursor.putBefore(keyThang, valueThang);
     }
 
     /**
@@ -631,10 +822,10 @@ final class DataCursor implements Cloneable {
      * time a putXxx() method is called that key will be used.
      */
     void useRangeKey() {
-        if (!range.singleKey) {
-            throw new IllegalStateException();
+        if (!range.isSingleKey()) {
+            throw DbCompat.unexpectedState();
         }
-        KeyRange.copy(range.beginKey, keyThang);
+        KeyRange.copy(range.getSingleKey(), keyThang);
     }
 
     /**
@@ -652,11 +843,11 @@ final class DataCursor implements Cloneable {
      */
     LockMode getLockMode(boolean lockForWrite) {
 
-        /* Dirty-read takes precedence over write-locking. */
+        /* Read-uncommmitted takes precedence over write-locking. */
 
-        if (dirtyRead) {
-            return LockMode.DIRTY_READ;
-        } else if (lockForWrite && !view.currentTxn.isDirtyRead()) {
+        if (readUncommitted) {
+            return LockMode.READ_UNCOMMITTED;
+        } else if (lockForWrite) {
             return view.currentTxn.getWriteLockMode();
         } else {
             return LockMode.DEFAULT;
