@@ -1,14 +1,14 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: ForeignKeyTrigger.java,v 1.1 2004/01/30 21:43:38 mark Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
+ *
+ */
 
 package com.sleepycat.je;
 
+import com.sleepycat.je.txn.Locker;
+
 class ForeignKeyTrigger implements DatabaseTrigger {
 
     private SecondaryDatabase secDb;
@@ -27,15 +27,14 @@ class ForeignKeyTrigger implements DatabaseTrigger {
     }
 
     public void databaseUpdated(Database db,
-                                Transaction txn,
+                                Locker locker,
                                 DatabaseEntry priKey,
                                 DatabaseEntry oldData,
                                 DatabaseEntry newData)
         throws DatabaseException {
 
         if (newData == null) {
-            secDb.onForeignKeyDelete((txn != null) ? txn.getLocker() : null, 
-                                      priKey);
+            secDb.onForeignKeyDelete(locker, priKey);
         }
     }
 }
