@@ -1,11 +1,10 @@
 /*
- * See the file LICENSE for redistribution iormation.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: BINDeltaLogEntry.java,v 1.14 2004/03/29 21:57:15 linda Exp $ 
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
+ *
+ */
+
 package com.sleepycat.je.log.entry;
 
 import com.sleepycat.je.DatabaseException;
@@ -13,26 +12,25 @@ import com.sleepycat.je.dbi.DatabaseId;
 import com.sleepycat.je.dbi.EnvironmentImpl;
 import com.sleepycat.je.tree.BINDelta;
 import com.sleepycat.je.tree.IN;
-import com.sleepycat.je.utilint.DbLsn;
 
 /**
  * A BINDeltaLogEntry knows how to create a whole BIN from a delta entry.
  */
-public class BINDeltaLogEntry extends SingleItemLogEntry
+public class BINDeltaLogEntry extends SingleItemEntry
     implements INContainingEntry {
 
     /**
      * @param logClass
      */
-    public BINDeltaLogEntry(Class logClass) {
+    public BINDeltaLogEntry(Class<BINDelta> logClass) {
         super(logClass);
     }
 
-    /* 
+    /*
      * @see com.sleepycat.je.log.entry.INContainingEntry#getIN()
      */
-    public IN getIN(EnvironmentImpl env) 
-    	throws DatabaseException {
+    public IN getIN(EnvironmentImpl env)
+            throws DatabaseException {
 
         BINDelta delta = (BINDelta) getMainItem();
         return delta.reconstituteBIN(env);
@@ -42,15 +40,17 @@ public class BINDeltaLogEntry extends SingleItemLogEntry
      * @see com.sleepycat.je.log.entry.INContainingEntry#getDbId()
      */
     public DatabaseId getDbId() {
+
         BINDelta delta = (BINDelta) getMainItem();
-        return delta.getDbId();	
+        return delta.getDbId();        
     }
 
     /**
-     * @return the lsn that represents this IN. For this BINDelta, it's
+     * @return the LSN that represents this IN. For this BINDelta, it's
      * the last full version.
      */
-    public DbLsn getLsnOfIN(DbLsn lastReadLsn) {
+    public long getLsnOfIN(long lastReadLsn) {
+
         BINDelta delta = (BINDelta) getMainItem();
         return delta.getLastFullLsn();
     }
