@@ -1,9 +1,8 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2008 Oracle.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: ComplexFormat.java,v 1.44.2.1 2008/09/29 16:28:57 mark Exp $
  */
 
 package com.sleepycat.persist.impl;
@@ -13,17 +12,18 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.IdentityHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import com.sleepycat.compat.DbCompat;
 import com.sleepycat.persist.evolve.Converter;
 import com.sleepycat.persist.evolve.Deleter;
 import com.sleepycat.persist.evolve.EntityConverter;
 import com.sleepycat.persist.evolve.Mutations;
 import com.sleepycat.persist.evolve.Renamer;
 import com.sleepycat.persist.model.ClassMetadata;
+import com.sleepycat.persist.model.DeleteAction;
 import com.sleepycat.persist.model.EntityMetadata;
 import com.sleepycat.persist.model.EntityModel;
 import com.sleepycat.persist.model.FieldMetadata;
@@ -31,6 +31,7 @@ import com.sleepycat.persist.model.Relationship;
 import com.sleepycat.persist.model.SecondaryKeyMetadata;
 import com.sleepycat.persist.raw.RawField;
 import com.sleepycat.persist.raw.RawObject;
+import com.sleepycat.je.utilint.IdentityHashMap;
 
 /**
  * Format for persistent complex classes that are not composite key classes.
@@ -49,14 +50,14 @@ public class ComplexFormat extends Format {
     private List<FieldInfo> nonKeyFields;
     private FieldReader secKeyFieldReader;
     private FieldReader nonKeyFieldReader;
-    private Map<String,String> oldToNewKeyMap;
-    private Map<String,String> newToOldFieldMap;
+    private Map<String, String> oldToNewKeyMap;
+    private Map<String, String> newToOldFieldMap;
     private boolean evolveNeeded;
     private transient Accessor objAccessor;
     private transient Accessor rawAccessor;
     private transient ComplexFormat entityFormat;
-    private transient Map<String,FieldAddress> secKeyAddresses;
-    private transient volatile Map<String,RawField> rawFields;
+    private transient Map<String, FieldAddress> secKeyAddresses;
+    private transient volatile Map<String, RawField> rawFields;
     private transient volatile FieldInfo[] rawInputFields;
     private transient volatile int[] rawInputLevels;
     private transient volatile int rawInputDepth;
@@ -118,6 +119,12 @@ public class ComplexFormat extends Format {
                              getClassName() + '.' + fieldName);
                     }
                 }
+                if (fieldCls.isPrimitive() &&
+                    secKeyMeta.getDeleteAction() == DeleteAction.NULLIFY) {
+                    throw new IllegalArgumentException
+                        ("NULLIFY may not be used with primitive fields: " +
+                         getClassName() + '.' + fieldName);
+                }
                 nonKeyFields.remove(field);
                 secKeyFields.add(field);
             }
@@ -129,7 +136,7 @@ public class ComplexFormat extends Format {
     }
 
     @Override
-    void migrateFromBeta(Map<String,Format> formatMap) {
+    void migrateFromBeta(Map<String, Format> formatMap) {
         super.migrateFromBeta(formatMap);
         if (priKeyField != null) {
             priKeyField.migrateFromBeta(formatMap);
@@ -146,7 +153,7 @@ public class ComplexFormat extends Format {
      * Returns getSuperFormat cast to ComplexFormat.  It is guaranteed that all
      * super formats of a ComplexFormat are a ComplexFormat.
      */
-    private ComplexFormat getComplexSuper() {
+    ComplexFormat getComplexSuper() {
         return (ComplexFormat) getSuperFormat();
     }
 
@@ -158,6 +165,10 @@ public class ComplexFormat extends Format {
         return (ComplexFormat) getLatestVersion();
     }
 
+    FieldInfo getPriKeyFieldInfo() {
+        return priKeyField;
+    }
+
     String getPriKeyField() {
         if (clsMeta.getPrimaryKey() != null) {
             return clsMeta.getPrimaryKey().getName();
@@ -199,7 +210,7 @@ public class ComplexFormat extends Format {
              * existing format which has its superFormat field set.
              */
             if (isNew()) {
-                throw new IllegalStateException(toString());
+                throw DbCompat.unexpectedState(toString());
             }
             for (ComplexFormat format = this;
                  format != null;
@@ -223,7 +234,7 @@ public class ComplexFormat extends Format {
     }
 
     @Override
-    public Map<String,RawField> getFields() {
+    public Map<String, RawField> getFields() {
 
         /*
          * Synchronization is not required since rawFields is immutable.  If
@@ -232,7 +243,7 @@ public class ComplexFormat extends Format {
          * the map is fully populated.
          */
         if (rawFields == null) {
-            Map<String,RawField> map = new HashMap<String,RawField>();
+            Map<String, RawField> map = new HashMap<String, RawField>();
             if (priKeyField != null) {
                 map.put(priKeyField.getName(), priKeyField);
             }
@@ -249,7 +260,7 @@ public class ComplexFormat extends Format {
 
     @Override
     void collectRelatedFormats(Catalog catalog,
-                               Map<String,Format> newFormats) {
+                               Map<String, Format> newFormats) {
         Class cls = getType();
         /* Collect field formats. */
         if (priKeyField != null) {
@@ -369,7 +380,7 @@ public class ComplexFormat extends Format {
         }
         /* Disallow primary keys on entity subclasses.  [#15757] */
         if (entityFormat != null &&
-            entityFormat != this && 
+            entityFormat != this &&
             priKeyField != null) {
             throw new IllegalArgumentException
                 ("A PrimaryKey may not appear on an Entity subclass: " +
@@ -399,7 +410,7 @@ public class ComplexFormat extends Format {
                 entityFormat.getLatestVersion().getEntityMetadata();
         }
         if (latestEntityMeta != null) {
-            secKeyAddresses = new HashMap<String,FieldAddress>();
+            secKeyAddresses = new HashMap<String, FieldAddress>();
             ComplexFormat thisLatest = getComplexLatest();
             if (thisLatest != this) {
                 thisLatest.initializeIfNeeded(catalog, model);
@@ -669,13 +680,23 @@ public class ComplexFormat extends Format {
         accessor.readPriKeyField(o, input);
     }
 
+    @Override
+    public String getOldKeyName(final String keyName) {
+        if (newToOldFieldMap != null &&
+            newToOldFieldMap.containsKey(keyName)) {
+            return newToOldFieldMap.get(keyName);
+        } else {
+            return keyName;
+        }
+    }
+
     @Override
     boolean nullifySecKey(Catalog catalog,
                           Object entity,
                           String keyName,
                           Object keyElement) {
         if (secKeyAddresses == null) {
-            throw new IllegalStateException();
+            throw DbCompat.unexpectedState();
         }
         FieldAddress addr = secKeyAddresses.get(keyName);
         if (addr != null) {
@@ -738,7 +759,7 @@ public class ComplexFormat extends Format {
     @Override
     Format skipToSecKey(RecordInput input, String keyName) {
         if (secKeyAddresses == null) {
-            throw new IllegalStateException();
+            throw DbCompat.unexpectedState();
         }
         FieldAddress addr = secKeyAddresses.get(keyName);
         if (addr != null) {
@@ -818,7 +839,7 @@ public class ComplexFormat extends Format {
         Mutations mutations = evolver.getMutations();
         boolean thisChanged = false;
         boolean hierarchyChanged = false;
-        Map<String,String> allKeyNameMap = new HashMap<String,String>();
+        Map<String, String> allKeyNameMap = new HashMap<String, String>();
 
         /* The Evolver has already ensured that entities evolve to entities. */
         assert isEntity() == newFormat.isEntity();
@@ -997,11 +1018,12 @@ public class ComplexFormat extends Format {
                 }
 
                 if (oldEntityClass != null && isCurrentVersion()) {
-                    Map<String,SecondaryKeyMetadata> secKeys =
+                    Map<String, SecondaryKeyMetadata> secKeys =
                         oldSuper.clsMeta.getSecondaryKeys();
                     for (FieldInfo field : oldSuper.secKeyFields) {
                         SecondaryKeyMetadata meta =
-                            secKeys.get(field.getName());
+                            getSecondaryKeyMetadataByFieldName
+                                (secKeys, field.getName());
                         assert meta != null;
                         allKeyNameMap.put(meta.getKeyName(), null);
                     }
@@ -1040,7 +1062,7 @@ public class ComplexFormat extends Format {
             oldEntityClass != null &&
             newEntityClass != null &&
             isCurrentVersion()) {
-            for (Map.Entry<String,String> entry : allKeyNameMap.entrySet()) {
+            for (Map.Entry<String, String> entry : allKeyNameMap.entrySet()) {
                 String oldKeyName = entry.getKey();
                 String newKeyName = entry.getValue();
                 if (newKeyName != null) {
@@ -1088,9 +1110,9 @@ public class ComplexFormat extends Format {
             deletedKeys = Collections.emptySet();
         }
 
-        Map<String,SecondaryKeyMetadata> oldSecondaryKeys =
+        Map<String, SecondaryKeyMetadata> oldSecondaryKeys =
             entityMeta.getSecondaryKeys();
-        Map<String,SecondaryKeyMetadata> newSecondaryKeys =
+        Map<String, SecondaryKeyMetadata> newSecondaryKeys =
             newFormat.entityMeta.getSecondaryKeys();
         Set<String> insertedKeys =
             new HashSet<String>(newSecondaryKeys.keySet());
@@ -1323,7 +1345,8 @@ public class ComplexFormat extends Format {
             String oldName = oldField.getName();
             SecondaryKeyMetadata oldMeta = null;
             if (isOldSecKeyField) {
-                oldMeta = clsMeta.getSecondaryKeys().get(oldName);
+                oldMeta = getSecondaryKeyMetadataByFieldName
+                    (clsMeta.getSecondaryKeys(), oldName);
                 assert oldMeta != null;
             }
 
@@ -1353,7 +1376,7 @@ public class ComplexFormat extends Format {
                 renamer.getNewName() : oldName;
             if (!oldName.equals(newName)) {
                 if (newToOldFieldMap == null) {
-                    newToOldFieldMap = new HashMap<String,String>();
+                    newToOldFieldMap = new HashMap<String, String>();
                 }
                 newToOldFieldMap.put(newName, oldName);
             }
@@ -1396,7 +1419,7 @@ public class ComplexFormat extends Format {
                 }
                 if (isOldSecKeyField) {
                     if (oldToNewKeyMap == null) {
-                        oldToNewKeyMap = new HashMap<String,String>();
+                        oldToNewKeyMap = new HashMap<String, String>();
                     }
                     oldToNewKeyMap.put(oldMeta.getKeyName(), null);
                 }
@@ -1421,7 +1444,8 @@ public class ComplexFormat extends Format {
             /* Get and process secondary key metadata changes. */
             SecondaryKeyMetadata newMeta = null;
             if (isOldSecKeyField && isNewSecKeyField) {
-                newMeta = newFormat.clsMeta.getSecondaryKeys().get(newName);
+                newMeta = getSecondaryKeyMetadataByFieldName
+                    (newFormat.clsMeta.getSecondaryKeys(), newName);
                 assert newMeta != null;
 
                 /* Validate metadata changes. */
@@ -1440,14 +1464,14 @@ public class ComplexFormat extends Format {
                 String newKeyName = newMeta.getKeyName();
                 if (!oldKeyName.equals(newKeyName)) {
                     if (oldToNewKeyMap == null) {
-                        oldToNewKeyMap = new HashMap<String,String>();
+                        oldToNewKeyMap = new HashMap<String, String>();
                     }
                     oldToNewKeyMap.put(oldName, newName);
                     localEvolveNeeded = true;
                 }
             } else if (isOldSecKeyField && !isNewSecKeyField) {
                 if (oldToNewKeyMap == null) {
-                    oldToNewKeyMap = new HashMap<String,String>();
+                    oldToNewKeyMap = new HashMap<String, String>();
                 }
                 oldToNewKeyMap.put(oldMeta.getKeyName(), null);
             }
@@ -1637,7 +1661,7 @@ public class ComplexFormat extends Format {
         }
 
         void addField(FieldInfo oldField) {
-            throw new UnsupportedOperationException();
+            throw DbCompat.unexpectedState();
         }
 
         abstract void readFields(Object o,
@@ -1777,7 +1801,7 @@ public class ComplexFormat extends Format {
                         ComplexFormat oldParentFormat,
                         ComplexFormat newParentFormat,
                         boolean isOldSecKey) {
-            
+
             /*
              * The oldFieldNum field was added as part of a bug fix.  If not
              * present in this version of the catalog, we assume it is equal to
@@ -2058,4 +2082,23 @@ public class ComplexFormat extends Format {
             return o;
         }
     }
+
+    /**
+     * The secondary key metadata map (ClassMetadata.getSecondaryKeys) is keyed
+     * by key name, not field name.  Key name can be different than field name
+     * when a @SecondaryKey name property is specified.  To look up metadata
+     * by field name, we must do a linear search.  Luckily, the number of keys
+     * per class is never very large. [#16819]
+     */
+    static SecondaryKeyMetadata
+        getSecondaryKeyMetadataByFieldName(Map<String, SecondaryKeyMetadata>
+                                             secKeys,
+                                           String fieldName) {
+        for (SecondaryKeyMetadata meta : secKeys.values()) {
+            if (meta.getName().equals(fieldName)) {
+                return meta;
+            }
+        }
+        return null;
+    }
 }
