@@ -1,20 +1,21 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: TupleBindingTest.java,v 1.32 2006/01/03 21:56:05 bostic Exp $
  */
 
 package com.sleepycat.bind.tuple.test;
 
+import java.math.BigInteger;
+
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
 import com.sleepycat.bind.EntityBinding;
 import com.sleepycat.bind.EntryBinding;
+import com.sleepycat.bind.tuple.BigIntegerBinding;
 import com.sleepycat.bind.tuple.BooleanBinding;
 import com.sleepycat.bind.tuple.ByteBinding;
 import com.sleepycat.bind.tuple.CharacterBinding;
@@ -23,6 +24,8 @@ import com.sleepycat.bind.tuple.FloatBinding;
 import com.sleepycat.bind.tuple.IntegerBinding;
 import com.sleepycat.bind.tuple.LongBinding;
 import com.sleepycat.bind.tuple.ShortBinding;
+import com.sleepycat.bind.tuple.SortedDoubleBinding;
+import com.sleepycat.bind.tuple.SortedFloatBinding;
 import com.sleepycat.bind.tuple.StringBinding;
 import com.sleepycat.bind.tuple.TupleBinding;
 import com.sleepycat.bind.tuple.TupleInput;
@@ -30,10 +33,10 @@ import com.sleepycat.bind.tuple.TupleInputBinding;
 import com.sleepycat.bind.tuple.TupleMarshalledBinding;
 import com.sleepycat.bind.tuple.TupleOutput;
 import com.sleepycat.bind.tuple.TupleTupleMarshalledBinding;
-import com.sleepycat.collections.test.DbTestUtil;
 import com.sleepycat.je.DatabaseEntry;
-import com.sleepycat.util.FastOutputStream;
 import com.sleepycat.util.ExceptionUnwrapper;
+import com.sleepycat.util.FastOutputStream;
+import com.sleepycat.util.test.SharedTestUtils;
 
 /**
  * @author Mark Hayes
@@ -43,9 +46,7 @@ public class TupleBindingTest extends TestCase {
     private DatabaseEntry buffer;
     private DatabaseEntry keyBuffer;
 
-    public static void main(String[] args)
-        throws Exception {
-
+    public static void main(String[] args) {
         junit.framework.TestResult tr =
             junit.textui.TestRunner.run(suite());
         if (tr.errorCount() > 0 ||
@@ -56,9 +57,7 @@ public class TupleBindingTest extends TestCase {
         }
     }
 
-    public static Test suite()
-        throws Exception {
-
+    public static Test suite() {
         TestSuite suite = new TestSuite(TupleBindingTest.class);
         return suite;
     }
@@ -68,13 +67,15 @@ public class TupleBindingTest extends TestCase {
         super(name);
     }
 
+    @Override
     public void setUp() {
 
-        DbTestUtil.printTestName("TupleBindingTest." + getName());
+        SharedTestUtils.printTestName("TupleBindingTest." + getName());
         buffer = new DatabaseEntry();
         keyBuffer = new DatabaseEntry();
     }
 
+    @Override
     public void tearDown() {
 
         /* Ensure that GC can cleanup. */
@@ -82,6 +83,7 @@ public class TupleBindingTest extends TestCase {
         keyBuffer = null;
     }
 
+    @Override
     public void runTest()
         throws Throwable {
 
@@ -114,20 +116,24 @@ public class TupleBindingTest extends TestCase {
         catch (ClassCastException expected) {}
 
         /* Test nested tuple binding. */
+        forMoreCoverageTest(binding, val);
+    }
+
+    private void forMoreCoverageTest(TupleBinding val1,Object val2) {
 
         TupleOutput output = new TupleOutput();
         output.writeString("abc");
-        binding.objectToEntry(val, output);
+        val1.objectToEntry(val2, output);
         output.writeString("xyz");
 
         TupleInput input = new TupleInput(output);
         assertEquals("abc", input.readString());
-        Object val3 = binding.entryToObject(input);
+        Object val3 = val1.entryToObject(input);
         assertEquals("xyz", input.readString());
 
         assertEquals(0, input.available());
-        assertSame(compareCls, val3.getClass());
-        assertEquals(val, val3);
+        assertSame(val2.getClass(), val3.getClass());
+        assertEquals(val2, val3);
     }
 
     public void testPrimitiveBindings() {
@@ -172,71 +178,103 @@ public class TupleBindingTest extends TestCase {
         DatabaseEntry entry = new DatabaseEntry();
 
         StringBinding.stringToEntry("abc", entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
         assertEquals("abc", StringBinding.entryToString(entry));
 
         new StringBinding().objectToEntry("abc", entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
 
         StringBinding.stringToEntry(null, entry);
-	assertEquals(2, entry.getData().length);
+        assertEquals(2, entry.getData().length);
         assertEquals(null, StringBinding.entryToString(entry));
 
         new StringBinding().objectToEntry(null, entry);
-	assertEquals(2, entry.getData().length);
+        assertEquals(2, entry.getData().length);
 
         CharacterBinding.charToEntry('a', entry);
-	assertEquals(2, entry.getData().length);
+        assertEquals(2, entry.getData().length);
         assertEquals('a', CharacterBinding.entryToChar(entry));
 
         new CharacterBinding().objectToEntry(new Character('a'), entry);
-	assertEquals(2, entry.getData().length);
+        assertEquals(2, entry.getData().length);
 
         BooleanBinding.booleanToEntry(true, entry);
-	assertEquals(1, entry.getData().length);
+        assertEquals(1, entry.getData().length);
         assertEquals(true, BooleanBinding.entryToBoolean(entry));
 
         new BooleanBinding().objectToEntry(Boolean.TRUE, entry);
-	assertEquals(1, entry.getData().length);
+        assertEquals(1, entry.getData().length);
 
         ByteBinding.byteToEntry((byte) 123, entry);
-	assertEquals(1, entry.getData().length);
+        assertEquals(1, entry.getData().length);
         assertEquals((byte) 123, ByteBinding.entryToByte(entry));
 
         ShortBinding.shortToEntry((short) 123, entry);
-	assertEquals(2, entry.getData().length);
+        assertEquals(2, entry.getData().length);
         assertEquals((short) 123, ShortBinding.entryToShort(entry));
 
         new ByteBinding().objectToEntry(new Byte((byte) 123), entry);
-	assertEquals(1, entry.getData().length);
+        assertEquals(1, entry.getData().length);
 
         IntegerBinding.intToEntry(123, entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
         assertEquals(123, IntegerBinding.entryToInt(entry));
 
         new IntegerBinding().objectToEntry(new Integer(123), entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
 
         LongBinding.longToEntry(123, entry);
-	assertEquals(8, entry.getData().length);
+        assertEquals(8, entry.getData().length);
         assertEquals(123, LongBinding.entryToLong(entry));
 
         new LongBinding().objectToEntry(new Long(123), entry);
-	assertEquals(8, entry.getData().length);
+        assertEquals(8, entry.getData().length);
 
         FloatBinding.floatToEntry((float) 123.123, entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
         assertTrue(((float) 123.123) == FloatBinding.entryToFloat(entry));
 
         new FloatBinding().objectToEntry(new Float((float) 123.123), entry);
-	assertEquals(4, entry.getData().length);
+        assertEquals(4, entry.getData().length);
 
         DoubleBinding.doubleToEntry(123.123, entry);
-	assertEquals(8, entry.getData().length);
+        assertEquals(8, entry.getData().length);
         assertTrue(123.123 == DoubleBinding.entryToDouble(entry));
 
         new DoubleBinding().objectToEntry(new Double(123.123), entry);
-	assertEquals(8, entry.getData().length);
+        assertEquals(8, entry.getData().length);
+
+        BigIntegerBinding.bigIntegerToEntry
+                (new BigInteger("1234567890123456"), entry);
+        assertEquals(9, entry.getData().length);
+        assertTrue((new BigInteger("1234567890123456")).equals
+                   (BigIntegerBinding.entryToBigInteger(entry)));
+
+        new BigIntegerBinding().objectToEntry
+                (new BigInteger("1234567890123456"), entry);
+        assertEquals(9, entry.getData().length);
+        forMoreCoverageTest(new BigIntegerBinding(),
+                            new BigInteger("1234567890123456"));
+
+        SortedFloatBinding.floatToEntry((float) 123.123, entry);
+        assertEquals(4, entry.getData().length);
+        assertTrue(((float) 123.123) ==
+                   SortedFloatBinding.entryToFloat(entry));
+
+        new SortedFloatBinding().objectToEntry
+            (new Float((float) 123.123), entry);
+        assertEquals(4, entry.getData().length);
+        forMoreCoverageTest(new SortedFloatBinding(),
+                            new Float((float) 123.123));
+
+        SortedDoubleBinding.doubleToEntry(123.123, entry);
+        assertEquals(8, entry.getData().length);
+        assertTrue(123.123 == SortedDoubleBinding.entryToDouble(entry));
+
+        new SortedDoubleBinding().objectToEntry(new Double(123.123), entry);
+        assertEquals(8, entry.getData().length);
+        forMoreCoverageTest(new SortedDoubleBinding(),
+                            new Double(123.123));
     }
 
     public void testTupleInputBinding() {
@@ -316,16 +354,19 @@ public class TupleBindingTest extends TestCase {
             super();
         }
 
+        @Override
         public TupleOutput getTupleOutput(Object object) {
             TupleOutput out = super.getTupleOutput(object);
             bufSize = out.getBufferBytes().length;
             return out;
         }
 
+        @Override
         public Object entryToObject(TupleInput input) {
             return input.readString();
         }
 
+        @Override
         public void objectToEntry(Object object, TupleOutput output) {
             assertEquals(bufSize, output.getBufferBytes().length);
             output.writeString((String) object);
@@ -363,16 +404,19 @@ public class TupleBindingTest extends TestCase {
             this.out = out;
         }
 
+        @Override
         public TupleOutput getTupleOutput(Object object) {
             out.reset();
             used = true;
             return out;
         }
 
+        @Override
         public Object entryToObject(TupleInput input) {
             return input.readString();
         }
 
+        @Override
         public void objectToEntry(Object object, TupleOutput output) {
             assertSame(out, output);
             output.writeString((String) object);
