@@ -1,15 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: KeyRangeTest.java,v 1.35 2006/01/03 21:56:05 bostic Exp $
  */
 
 package com.sleepycat.collections;
 
 import java.io.File;
+import java.io.Serializable;
 import java.util.Arrays;
 import java.util.Comparator;
 
@@ -18,7 +17,6 @@ import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
 import com.sleepycat.bind.ByteArrayBinding;
-import com.sleepycat.collections.test.DbTestUtil;
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
@@ -27,6 +25,9 @@ import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.OperationStatus;
+import com.sleepycat.util.keyrange.KeyRange;
+import com.sleepycat.util.keyrange.KeyRangeException;
+import com.sleepycat.util.test.SharedTestUtils;
 
 /**
  * @author Mark Hayes
@@ -57,9 +58,7 @@ public class KeyRangeTest extends TestCase {
     private DataView view;
     private DataCursor cursor;
 
-    public static void main(String[] args)
-        throws Exception {
-
+    public static void main(String[] args) {
         junit.framework.TestResult tr =
             junit.textui.TestRunner.run(suite());
         if (tr.errorCount() > 0 ||
@@ -80,16 +79,15 @@ public class KeyRangeTest extends TestCase {
         super(name);
     }
 
-    public void setUp()
-        throws Exception {
-
-        DbTestUtil.printTestName(DbTestUtil.qualifiedTestName(this));
+    @Override
+    public void setUp() {
+        SharedTestUtils.printTestName(SharedTestUtils.qualifiedTestName(this));
     }
 
-    private void openDb(Comparator comparator)
+    private void openDb(Comparator<byte []> comparator)
         throws Exception {
 
-        File dir = DbTestUtil.getNewDir();
+        File dir = SharedTestUtils.getNewDir();
         ByteArrayBinding dataBinding = new ByteArrayBinding();
         EnvironmentConfig envConfig = new EnvironmentConfig();
         envConfig.setAllowCreate(true);
@@ -98,25 +96,25 @@ public class KeyRangeTest extends TestCase {
         DatabaseConfig dbConfig = new DatabaseConfig();
         DbCompat.setTypeBtree(dbConfig);
         dbConfig.setAllowCreate(true);
-	if (comparator != null) {
-	    DbCompat.setBtreeComparator(dbConfig, comparator);
-	}
-        store = DbCompat.openDatabase(env, null, "test.db", null, dbConfig);
+        if (comparator != null) {
+            DbCompat.setBtreeComparator(dbConfig, comparator);
+        }
+        store = DbCompat.testOpenDatabase
+            (env, null, "test.db", null, dbConfig);
         view = new DataView(store, dataBinding, dataBinding, null, true, null);
     }
 
     private void closeDb()
         throws Exception {
-        
+
         store.close();
         store = null;
         env.close();
         env = null;
     }
 
-    public void tearDown()
-        throws Exception {
-
+    @Override
+    public void tearDown() {
         try {
             if (store != null) {
                 store.close();
@@ -370,8 +368,8 @@ public class KeyRangeTest extends TestCase {
         DatabaseEntry end = new DatabaseEntry();
         DatabaseEntry end2 = new DatabaseEntry();
         KeyRange range = new KeyRange(null);
-        KeyRange range2;
-
+        KeyRange range2 = null;
+        
         /* Base range [1, 2] */
         begin.setData(new byte[] { 1 });
         end.setData(new byte[] { 2 });
@@ -423,10 +421,10 @@ public class KeyRangeTest extends TestCase {
         } catch (KeyRangeException expected) {}
     }
 
-    public static class ReverseComparator implements Comparator {
-        public int compare(Object o1, Object o2) {
-            byte[] d1 = (byte[]) o1;
-            byte[] d2 = (byte[]) o2;
+    @SuppressWarnings("serial")
+    public static class ReverseComparator implements Comparator<byte[]>,
+                                                     Serializable {
+        public int compare(byte[] d1, byte[] d2) {
             int cmp = KeyRange.compareBytes(d1, 0, d1.length,
                                             d2, 0, d2.length);
             if (cmp < 0) {
