@@ -1,17 +1,14 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: TestKeyCreator.java,v 1.22 2006/01/03 21:56:07 bostic Exp $
  */
 
 package com.sleepycat.collections.test;
 
 import com.sleepycat.bind.RecordNumberBinding;
 import com.sleepycat.je.DatabaseEntry;
-import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.SecondaryDatabase;
 import com.sleepycat.je.SecondaryKeyCreator;
 
@@ -21,7 +18,7 @@ import com.sleepycat.je.SecondaryKeyCreator;
  */
 class TestKeyCreator implements SecondaryKeyCreator {
 
-    private boolean isRecNum;
+    private final boolean isRecNum;
 
     TestKeyCreator(boolean isRecNum) {
 
@@ -31,9 +28,7 @@ class TestKeyCreator implements SecondaryKeyCreator {
     public boolean createSecondaryKey(SecondaryDatabase db,
                                       DatabaseEntry primaryKeyData,
                                       DatabaseEntry valueData,
-                                      DatabaseEntry indexKeyData)
-        throws DatabaseException {
-
+                                      DatabaseEntry indexKeyData) {
         if (valueData.getSize() == 0) {
             return false;
         }
