@@ -1,15 +1,12 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: TestStore.java,v 1.38 2006/01/03 21:56:07 bostic Exp $
  */
 
 package com.sleepycat.collections.test;
 
-import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -213,6 +210,10 @@ class TestStore {
         return DbCompat.isTypeQueue(config) || DbCompat.isTypeRecno(config);
     }
 
+    boolean areKeyRangesAllowed() {
+        return isOrdered() && !isQueueOrRecno();
+    }
+
     boolean areDuplicatesAllowed() {
 
         return DbCompat.getSortedDuplicates(config) ||
@@ -237,14 +238,14 @@ class TestStore {
     }
 
     Database open(Environment env, String fileName)
-        throws IOException, DatabaseException {
+        throws DatabaseException {
 
         int fixedLen = (isQueueOrRecno() ? 1 : 0);
         return openDb(env, fileName, fixedLen, null);
     }
 
     Database openIndex(Database primary, String fileName)
-        throws IOException, DatabaseException {
+        throws DatabaseException {
 
         int fixedLen = (isQueueOrRecno() ? 4 : 0);
         config.setKeyCreator(isRecNumFormat ? RECNO_EXTRACTOR
@@ -255,7 +256,7 @@ class TestStore {
 
     private Database openDb(Environment env, String fileName, int fixedLen,
                             Database primary)
-        throws IOException, DatabaseException {
+        throws DatabaseException {
 
         if (fixedLen > 0) {
             DbCompat.setRecordLength(config, fixedLen);
@@ -267,13 +268,11 @@ class TestStore {
         DbCompat.setReadUncommitted(config, true);
         config.setTransactional(CurrentTransaction.getInstance(env) != null);
         if (primary != null) {
-            return DbCompat.openSecondaryDatabase(env, null, 
-                                                  fileName, null,
-                                                  primary, config);
+            return DbCompat.testOpenSecondaryDatabase
+                (env, null, fileName, null, primary, config);
         } else {
-            return DbCompat.openDatabase(env, null, 
-                                         fileName, null,
-                                         config);
+            return DbCompat.testOpenDatabase
+                (env, null, fileName, null, config);
         }
     }
 }
