@@ -1,17 +1,18 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2004
- *	Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: TransactionTest.java,v 1.37 2004/04/09 12:42:38 mark Exp $
  */
 
 package com.sleepycat.collections.test;
 
+import java.io.File;
+import java.io.FileNotFoundException;
 import java.util.Iterator;
 import java.util.List;
 import java.util.SortedSet;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import junit.framework.Test;
 import junit.framework.TestCase;
@@ -26,12 +27,21 @@ import com.sleepycat.collections.StoredSortedMap;
 import com.sleepycat.collections.TransactionRunner;
 import com.sleepycat.collections.TransactionWorker;
 import com.sleepycat.compat.DbCompat;
+import com.sleepycat.je.Cursor;
+import com.sleepycat.je.CursorConfig;
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
+import com.sleepycat.je.DatabaseEntry;
+import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.Environment;
+import com.sleepycat.je.EnvironmentConfig;
+import com.sleepycat.je.LockConflictException;
+import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.TransactionConfig;
 import com.sleepycat.util.RuntimeExceptionWrapper;
+import com.sleepycat.util.test.SharedTestUtils;
+import com.sleepycat.util.test.TestEnv;
 
 /**
  * @author Mark Hayes
@@ -46,9 +56,7 @@ public class TransactionTest extends TestCase {
      * Runs a command line collection test.
      * @see #usage
      */
-    public static void main(String[] args)
-        throws Exception {
-
+    public static void main(String[] args) {
         if (args.length == 1 &&
             (args[0].equals("-h") || args[0].equals("-help"))) {
             usage();
@@ -72,9 +80,7 @@ public class TransactionTest extends TestCase {
         System.exit(2);
     }
 
-    public static Test suite()
-        throws Exception {
-
+    public static Test suite() {
         TestSuite suite = new TestSuite(TransactionTest.class);
         return suite;
     }
@@ -90,10 +96,11 @@ public class TransactionTest extends TestCase {
         super(name);
     }
 
+    @Override
     public void setUp()
         throws Exception {
 
-        DbTestUtil.printTestName(DbTestUtil.qualifiedTestName(this));
+        SharedTestUtils.printTestName(SharedTestUtils.qualifiedTestName(this));
         env = TestEnv.TXN.open("TransactionTests");
         currentTxn = CurrentTransaction.getInstance(env);
         store = testStore.open(env, dbName(0));
@@ -101,6 +108,7 @@ public class TransactionTest extends TestCase {
                                   testStore.getValueBinding(), true);
     }
 
+    @Override
     public void tearDown() {
 
         try {
@@ -144,23 +152,43 @@ public class TransactionTest extends TestCase {
         currentTxn.abortTransaction();
         assertNull(currentTxn.getTransaction());
 
-        // dirty-read property should be inherited
+        // read-uncommitted property should be inherited
 
-        assertTrue(!map.isDirtyRead());
-        assertTrue(!((StoredContainer) map.values()).isDirtyRead());
-        assertTrue(!((StoredContainer) map.keySet()).isDirtyRead());
-        assertTrue(!((StoredContainer) map.entrySet()).isDirtyRead());
+        assertTrue(!isReadUncommitted(map));
+        assertTrue(!isReadUncommitted(map.values()));
+        assertTrue(!isReadUncommitted(map.keySet()));
+        assertTrue(!isReadUncommitted(map.entrySet()));
 
-        StoredSortedMap other =
-            (StoredSortedMap) StoredCollections.dirtyReadMap(map);
-        assertTrue(other.isDirtyRead());
-        assertTrue(((StoredContainer) other.values()).isDirtyRead());
-        assertTrue(((StoredContainer) other.keySet()).isDirtyRead());
-        assertTrue(((StoredContainer) other.entrySet()).isDirtyRead());
-        assertTrue(!map.isDirtyRead());
-        assertTrue(!((StoredContainer) map.values()).isDirtyRead());
-        assertTrue(!((StoredContainer) map.keySet()).isDirtyRead());
-        assertTrue(!((StoredContainer) map.entrySet()).isDirtyRead());
+        StoredSortedMap other = (StoredSortedMap)
+            StoredCollections.configuredMap
+                (map, CursorConfig.READ_UNCOMMITTED);
+        assertTrue(isReadUncommitted(other));
+        assertTrue(isReadUncommitted(other.values()));
+        assertTrue(isReadUncommitted(other.keySet()));
+        assertTrue(isReadUncommitted(other.entrySet()));
+        assertTrue(!isReadUncommitted(map));
+        assertTrue(!isReadUncommitted(map.values()));
+        assertTrue(!isReadUncommitted(map.keySet()));
+        assertTrue(!isReadUncommitted(map.entrySet()));
+
+        // read-committed property should be inherited
+
+        assertTrue(!isReadCommitted(map));
+        assertTrue(!isReadCommitted(map.values()));
+        assertTrue(!isReadCommitted(map.keySet()));
+        assertTrue(!isReadCommitted(map.entrySet()));
+
+        other = (StoredSortedMap)
+            StoredCollections.configuredMap
+                (map, CursorConfig.READ_COMMITTED);
+        assertTrue(isReadCommitted(other));
+        assertTrue(isReadCommitted(other.values()));
+        assertTrue(isReadCommitted(other.keySet()));
+        assertTrue(isReadCommitted(other.entrySet()));
+        assertTrue(!isReadCommitted(map));
+        assertTrue(!isReadCommitted(map.values()));
+        assertTrue(!isReadCommitted(map.keySet()));
+        assertTrue(!isReadCommitted(map.entrySet()));
     }
 
     public void testTransactional()
@@ -178,9 +206,8 @@ public class TransactionTest extends TestCase {
         DatabaseConfig dbConfig = new DatabaseConfig();
         DbCompat.setTypeBtree(dbConfig);
         dbConfig.setAllowCreate(true);
-        Database db = DbCompat.openDatabase(env, null, 
-                                            dbName(1), null,
-                                            dbConfig);
+        Database db = DbCompat.testOpenDatabase
+            (env, null, dbName(1), null, dbConfig);
         map = new StoredSortedMap(db, testStore.getKeyBinding(),
                                       testStore.getValueBinding(), true);
         assertTrue(!map.isTransactional());
@@ -192,8 +219,8 @@ public class TransactionTest extends TestCase {
         //
         dbConfig.setTransactional(true);
         currentTxn.beginTransaction(null);
-        db = DbCompat.openDatabase(env, currentTxn.getTransaction(), 
-                                   dbName(2), null, dbConfig);
+        db = DbCompat.testOpenDatabase
+            (env, currentTxn.getTransaction(), dbName(2), null, dbConfig);
         currentTxn.commitTransaction();
         map = new StoredSortedMap(db, testStore.getKeyBinding(),
                                       testStore.getValueBinding(), true);
@@ -409,66 +436,234 @@ public class TransactionTest extends TestCase {
         assertNull(currentTxn.getTransaction());
     }
 
-    public void testDirtyReadCollection()
+    public void testReadCommittedCollection()
         throws Exception {
 
-        StoredSortedMap dirtyMap =
-            (StoredSortedMap) StoredCollections.dirtyReadSortedMap(map);
+        StoredSortedMap degree2Map = (StoredSortedMap)
+            StoredCollections.configuredSortedMap
+                (map, CursorConfig.READ_COMMITTED);
 
-        // original map is not dirty-read
-        assertTrue(map.isDirtyReadAllowed());
-        assertTrue(!map.isDirtyRead());
+        // original map is not read-committed
+        assertTrue(!isReadCommitted(map));
 
-        // all dirty-read containers are dirty-read
-        checkDirtyReadProperty(dirtyMap);
-        checkDirtyReadProperty(StoredCollections.dirtyReadMap(map));
-        checkDirtyReadProperty(StoredCollections.dirtyReadCollection(
-                                map.values()));
-        checkDirtyReadProperty(StoredCollections.dirtyReadSet(
-                                map.keySet()));
-        checkDirtyReadProperty(StoredCollections.dirtyReadSortedSet(
-                                (SortedSet) map.keySet()));
+        // all read-committed containers are read-uncommitted
+        assertTrue(isReadCommitted(degree2Map));
+        assertTrue(isReadCommitted
+            (StoredCollections.configuredMap
+                (map, CursorConfig.READ_COMMITTED)));
+        assertTrue(isReadCommitted
+            (StoredCollections.configuredCollection
+                (map.values(), CursorConfig.READ_COMMITTED)));
+        assertTrue(isReadCommitted
+            (StoredCollections.configuredSet
+                (map.keySet(), CursorConfig.READ_COMMITTED)));
+        assertTrue(isReadCommitted
+            (StoredCollections.configuredSortedSet
+                ((SortedSet) map.keySet(),
+                 CursorConfig.READ_COMMITTED)));
 
         if (DbCompat.RECNO_METHOD) {
-            // create a list just so we can call dirtyReadList()
-            Database listStore = TestStore.RECNO_RENUM.open(env, null);
+            // create a list just so we can call configuredList()
+            Database listStore = TestStore.RECNO_RENUM.open(env, "foo");
             List list = new StoredList(listStore, TestStore.VALUE_BINDING,
                                        true);
-            checkDirtyReadProperty(StoredCollections.dirtyReadList(list));
+            assertTrue(isReadCommitted
+                (StoredCollections.configuredList
+                    (list, CursorConfig.READ_COMMITTED)));
             listStore.close();
         }
 
-        doDirtyRead(dirtyMap);
+        map.put(ONE, ONE);
+        doReadCommitted(degree2Map, null);
     }
 
-    private void checkDirtyReadProperty(Object container) {
-
-        assertTrue(((StoredContainer) container).isDirtyReadAllowed());
-        assertTrue(((StoredContainer) container).isDirtyRead());
+    private static boolean isReadCommitted(Object container) {
+        StoredContainer storedContainer = (StoredContainer) container;
+        /* We can't use getReadCommitted until is is added to DB core. */
+        return storedContainer.getCursorConfig() != null &&
+               storedContainer.getCursorConfig().getReadCommitted();
     }
 
-    public void testDirtyReadTransaction()
+    public void testReadCommittedTransaction()
         throws Exception {
 
-        TransactionRunner runner = new TransactionRunner(env);
         TransactionConfig config = new TransactionConfig();
-        config.setDirtyRead(true);
-        runner.setTransactionConfig(config);
+        config.setReadCommitted(true);
+        doReadCommitted(map, config);
+    }
+
+    private void doReadCommitted(final StoredSortedMap degree2Map,
+                                 TransactionConfig txnConfig)
+        throws Exception {
+
+        map.put(ONE, ONE);
+        TransactionRunner runner = new TransactionRunner(env);
+        runner.setTransactionConfig(txnConfig);
         assertNull(currentTxn.getTransaction());
         runner.run(new TransactionWorker() {
             public void doWork() throws Exception {
                 assertNotNull(currentTxn.getTransaction());
-                doDirtyRead(map);
+
+                /* Do a read-committed get(), the lock is not retained. */
+                assertEquals(ONE, degree2Map.get(ONE));
+
+                /*
+                 * If we were not using read-committed, the following write of
+                 * key ONE with an auto-commit transaction would self-deadlock
+                 * since two transactions in the same thread would be
+                 * attempting to lock the same key, one for write and one for
+                 * read.  This test passes if we do not deadlock.
+                 */
+                DatabaseEntry key = new DatabaseEntry();
+                DatabaseEntry value = new DatabaseEntry();
+                testStore.getKeyBinding().objectToEntry(ONE, key);
+                testStore.getValueBinding().objectToEntry(TWO, value);
+                store.put(null, key, value);
             }
         });
         assertNull(currentTxn.getTransaction());
     }
 
-    private synchronized void doDirtyRead(StoredSortedMap dirtyMap)
+    public void testReadUncommittedCollection()
+        throws Exception {
+
+        StoredSortedMap dirtyMap = (StoredSortedMap)
+            StoredCollections.configuredSortedMap
+                (map, CursorConfig.READ_UNCOMMITTED);
+
+        // original map is not read-uncommitted
+        assertTrue(!isReadUncommitted(map));
+
+        // all read-uncommitted containers are read-uncommitted
+        assertTrue(isReadUncommitted(dirtyMap));
+        assertTrue(isReadUncommitted
+            (StoredCollections.configuredMap
+                (map, CursorConfig.READ_UNCOMMITTED)));
+        assertTrue(isReadUncommitted
+            (StoredCollections.configuredCollection
+                (map.values(), CursorConfig.READ_UNCOMMITTED)));
+        assertTrue(isReadUncommitted
+            (StoredCollections.configuredSet
+                (map.keySet(), CursorConfig.READ_UNCOMMITTED)));
+        assertTrue(isReadUncommitted
+            (StoredCollections.configuredSortedSet
+                ((SortedSet) map.keySet(), CursorConfig.READ_UNCOMMITTED)));
+
+        if (DbCompat.RECNO_METHOD) {
+            // create a list just so we can call configuredList()
+            Database listStore = TestStore.RECNO_RENUM.open(env, "foo");
+            List list = new StoredList(listStore, TestStore.VALUE_BINDING,
+                                       true);
+            assertTrue(isReadUncommitted
+                (StoredCollections.configuredList
+                    (list, CursorConfig.READ_UNCOMMITTED)));
+            listStore.close();
+        }
+
+        doReadUncommitted(dirtyMap);
+    }
+
+    private static boolean isReadUncommitted(Object container) {
+        StoredContainer storedContainer = (StoredContainer) container;
+        return storedContainer.getCursorConfig() != null &&
+               storedContainer.getCursorConfig().getReadUncommitted();
+    }
+
+    public void testReadUncommittedTransaction()
+        throws Exception {
+
+        TransactionRunner runner = new TransactionRunner(env);
+        TransactionConfig config = new TransactionConfig();
+        config.setReadUncommitted(true);
+        runner.setTransactionConfig(config);
+        assertNull(currentTxn.getTransaction());
+        runner.run(new TransactionWorker() {
+            public void doWork() throws Exception {
+                assertNotNull(currentTxn.getTransaction());
+                doReadUncommitted(map);
+            }
+        });
+        assertNull(currentTxn.getTransaction());
+    }
+
+    /**
+     * Tests that the CurrentTransaction static WeakHashMap does indeed allow
+     * GC to reclaim tine environment when it is closed.  At one point this was
+     * not working because the value object in the map has a reference to the
+     * environment.  This was fixed by wrapping the Environment in a
+     * WeakReference.  [#15444]
+     *
+     * This test only succeeds intermittently, probably due to its reliance
+     * on the GC call.
+     */
+    public void testCurrentTransactionGC()
+        throws Exception {
+
+        /*
+         * This test can have indeterminate results because it depends on
+         * a finalize count, so it's not part of the default run.
+         */
+        if (!SharedTestUtils.runLongTests()) {
+            return;
+        }
+
+        final StringBuffer finalizedFlag = new StringBuffer();
+
+        class MyEnv extends Environment {
+
+            /**
+             * @throws FileNotFoundException from DB core.
+             */
+            MyEnv(File home, EnvironmentConfig config)
+                throws DatabaseException, FileNotFoundException {
+
+                super(home, config);
+            }
+
+            @Override
+            protected void finalize() {
+                finalizedFlag.append('.');
+            }
+        }
+
+        MyEnv myEnv = new MyEnv(env.getHome(), env.getConfig());
+        CurrentTransaction myCurrTxn = CurrentTransaction.getInstance(myEnv);
+
+        store.close();
+        store = null;
+        map = null;
+
+        env.close();
+        env = null;
+
+        myEnv.close();
+        myEnv = null;
+
+        myCurrTxn = null;
+        currentTxn = null;
+
+        for (int i = 0; i < 10; i += 1) {
+            byte[] x = null;
+            try {
+                 x = new byte[Integer.MAX_VALUE - 1];
+            } catch (OutOfMemoryError expected) {
+            }
+            assertNull(x);
+            System.gc();
+        }
+
+        for (int i = 0; i < 10; i += 1) {
+            System.gc();
+        }
+
+        assertTrue(finalizedFlag.length() > 0);
+    }
+
+    private synchronized void doReadUncommitted(StoredSortedMap dirtyMap)
         throws Exception {
 
         // start thread one
-        DirtyReadThreadOne t1 = new DirtyReadThreadOne(env, this);
+        ReadUncommittedThreadOne t1 = new ReadUncommittedThreadOne(env, this);
         t1.start();
         wait();
 
@@ -485,7 +680,7 @@ public class TransactionTest extends TestCase {
         assertTrue(dirtyMap.isEmpty());
 
         // start thread two
-        DirtyReadThreadTwo t2 = new DirtyReadThreadTwo(env, this);
+        ReadUncommittedThreadTwo t2 = new ReadUncommittedThreadTwo(env, this);
         t2.start();
         wait();
 
@@ -502,21 +697,21 @@ public class TransactionTest extends TestCase {
         assertTrue(!dirtyMap.isEmpty());
     }
 
-    private static class DirtyReadThreadOne extends Thread {
+    private static class ReadUncommittedThreadOne extends Thread {
 
-        private Environment env;
-        private CurrentTransaction currentTxn;
-        private TransactionTest parent;
-        private StoredSortedMap map;
+        private final CurrentTransaction currentTxn;
+        private final TransactionTest parent;
+        private final StoredSortedMap map;
 
-        private DirtyReadThreadOne(Environment env, TransactionTest parent) {
+        private ReadUncommittedThreadOne(Environment env,
+                                         TransactionTest parent) {
 
-            this.env = env;
             this.currentTxn = CurrentTransaction.getInstance(env);
             this.parent = parent;
             this.map = parent.map;
         }
 
+        @Override
         public synchronized void run() {
 
             try {
@@ -542,14 +737,15 @@ public class TransactionTest extends TestCase {
         }
     }
 
-    private static class DirtyReadThreadTwo extends Thread {
+    private static class ReadUncommittedThreadTwo extends Thread {
 
-        private Environment env;
-        private CurrentTransaction currentTxn;
-        private TransactionTest parent;
-        private StoredSortedMap map;
+        private final Environment env;
+        private final CurrentTransaction currentTxn;
+        private final TransactionTest parent;
+        private final StoredSortedMap map;
 
-        private DirtyReadThreadTwo(Environment env, TransactionTest parent) {
+        private ReadUncommittedThreadTwo(Environment env,
+                                         TransactionTest parent) {
 
             this.env = env;
             this.currentTxn = CurrentTransaction.getInstance(env);
@@ -557,6 +753,7 @@ public class TransactionTest extends TestCase {
             this.map = parent.map;
         }
 
+        @Override
         public synchronized void run() {
 
             try {
@@ -623,4 +820,180 @@ public class TransactionTest extends TestCase {
             finally { StoredIterator.close(i); }
         }
     }
+
+    /**
+     * Tests transaction retries performed by TransationRunner.
+     *
+     * This test is too sensitive to how lock conflict detection works on JE to
+     * make it work properly on DB core.
+     */
+    /* <!-- begin JE only --> */
+    public void testRetry()
+        throws Exception {
+
+        final AtomicInteger tries = new AtomicInteger();
+        final AtomicInteger releaseLockAfterTries = new AtomicInteger();
+        final Transaction txn1 = env.beginTransaction(null, null);
+        final Cursor txn1Cursor =
+            store.openCursor(txn1, CursorConfig.READ_COMMITTED);
+        final TransactionRunner runner = new TransactionRunner(env);
+
+        final TransactionWorker worker = new TransactionWorker() {
+            public void doWork() throws Exception {
+                tries.getAndIncrement();
+                if (releaseLockAfterTries.get() == tries.get()) {
+                    /* With READ_COMMITTED, getNext releases the lock. */
+                    txn1Cursor.getNext(new DatabaseEntry(),
+                                       new DatabaseEntry(), null);
+                }
+                Transaction txn2 = currentTxn.getTransaction();
+                assertNotNull(txn2);
+                txn2.setLockTimeout(10 * 1000); /* Speed up the test. */
+                assertTrue(txn1 != txn2);
+                map.put(ONE, TWO);
+            }
+        };
+
+        /* Insert ONE and TWO with auto-commit.  Leave no records locked. */
+        DatabaseEntry key = new DatabaseEntry();
+        DatabaseEntry value = new DatabaseEntry();
+
+        testStore.getKeyBinding().objectToEntry(ONE, key);
+        testStore.getValueBinding().objectToEntry(ONE, value);
+        OperationStatus status = store.put(null, key, value);
+        assertSame(OperationStatus.SUCCESS, status);
+
+        testStore.getKeyBinding().objectToEntry(TWO, key);
+        testStore.getValueBinding().objectToEntry(TWO, value);
+        status = store.put(null, key, value);
+        assertSame(OperationStatus.SUCCESS, status);
+
+        /*
+         * Disable timouts in txn1 so that the timeout will occur in txn2 (in
+         * TransactionRunner).
+         */
+        txn1.setLockTimeout(0);
+
+        /*
+         * Read ONE with txn1 and leave it locked.  Expect the default number
+         * of retries and then a lock conflict.
+         */
+        status = txn1Cursor.getFirst(key, value, null);
+        assertSame(OperationStatus.SUCCESS, status);
+        int expectTries = TransactionRunner.DEFAULT_MAX_RETRIES + 1;
+        releaseLockAfterTries.set(0);
+        tries.set(0);
+        try {
+            runner.run(worker);
+            fail();
+        } catch (LockConflictException expected) {
+        }
+        assertEquals(expectTries, tries.get());
+
+        /* Same as above but use a custom number of retries. */
+        status = txn1Cursor.getFirst(key, value, null);
+        assertSame(OperationStatus.SUCCESS, status);
+        expectTries = 5;
+        runner.setMaxRetries(expectTries - 1);
+        releaseLockAfterTries.set(0);
+        tries.set(0);
+        try {
+            runner.run(worker);
+            fail();
+        } catch (LockConflictException expected) {
+        }
+        assertEquals(expectTries, tries.get());
+
+        /*
+         * In this variant the TransactionWorker will move the txn1Cursor
+         * forward to free the lock after 3 tries.  The 4th try will succeed,
+         * so do not expect a lock conflict.
+         */
+        status = txn1Cursor.getFirst(key, value, null);
+        assertSame(OperationStatus.SUCCESS, status);
+        expectTries = 3;
+        releaseLockAfterTries.set(expectTries);
+        tries.set(0);
+        runner.run(worker);
+        assertEquals(expectTries, tries.get());
+
+        /* Cleanup. */
+        txn1Cursor.close();
+        txn1.abort();
+    }
+    /* <!-- end JE only --> */
+
+    /**
+     * Tests transaction retries performed by TransationRunner.
+     *
+     * This test is too sensitive to how lock conflict detection works on JE to
+     * make it work properly on DB core.
+     */
+    /* <!-- begin JE only --> */
+    public void testExceptionHandler()
+        throws Exception {
+
+        class RetriesExceeded extends Exception {}
+        final int customMaxRetries = TransactionRunner.DEFAULT_MAX_RETRIES * 2;
+
+        final TransactionRunner runner = new TransactionRunner(env) {
+            @Override
+            public int handleException(Exception e,
+                                       int retries,
+                                       int maxRetries)
+                throws Exception {
+                if (e instanceof LockConflictException) {
+                    if (retries >= maxRetries) {
+                        throw new RetriesExceeded();
+                    } else {
+                        return customMaxRetries;
+                    }
+                } else {
+                    throw e;
+                }
+            }
+        };
+
+        final Transaction txn1 = env.beginTransaction(null, null);
+        final AtomicInteger tries = new AtomicInteger();
+
+        final TransactionWorker worker = new TransactionWorker() {
+            public void doWork() throws Exception {
+                tries.getAndIncrement();
+                Transaction txn2 = currentTxn.getTransaction();
+                assertNotNull(txn2);
+                txn2.setLockTimeout(10 * 1000); /* Speed up the test. */
+                assertTrue(txn1 != txn2);
+                map.put(ONE, TWO);
+            }
+        };
+
+        /* Insert ONE with txn1.  Leave the record locked. */
+        DatabaseEntry key = new DatabaseEntry();
+        DatabaseEntry value = new DatabaseEntry();
+
+        testStore.getKeyBinding().objectToEntry(ONE, key);
+        testStore.getValueBinding().objectToEntry(ONE, value);
+        OperationStatus status = store.put(txn1, key, value);
+        assertSame(OperationStatus.SUCCESS, status);
+
+        /*
+         * Disable timouts in txn1 so that the timeout will occur in txn2 (in
+         * TransactionRunner).
+         */
+        txn1.setLockTimeout(0);
+
+        /* Expect the custom number of retries and the custom exception. */
+        tries.set(0);
+        try {
+            runner.run(worker);
+            fail();
+        } catch (RetriesExceeded expected) {
+        }
+        assertEquals(customMaxRetries + 1, tries.get());
+
+        /* Cleanup. */
+        txn1.abort();
+    }
+    /* <!-- end JE only --> */
 }
