@@ -1,10 +1,8 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2000-2010 Oracle.  All rights reserved.
  *
- * $Id: TupleSerialFactoryTest.java,v 1.37 2006/01/03 21:56:10 bostic Exp $
  */
 package com.sleepycat.collections.test.serial;
 
@@ -19,8 +17,6 @@ import com.sleepycat.bind.serial.test.MarshalledObject;
 import com.sleepycat.collections.TransactionRunner;
 import com.sleepycat.collections.TransactionWorker;
 import com.sleepycat.collections.TupleSerialFactory;
-import com.sleepycat.collections.test.DbTestUtil;
-import com.sleepycat.collections.test.TestEnv;
 import com.sleepycat.compat.DbCompat;
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
@@ -28,6 +24,8 @@ import com.sleepycat.je.Environment;
 import com.sleepycat.je.ForeignKeyDeleteAction;
 import com.sleepycat.je.SecondaryConfig;
 import com.sleepycat.je.SecondaryDatabase;
+import com.sleepycat.util.test.SharedTestUtils;
+import com.sleepycat.util.test.TestEnv;
 
 /**
  * @author Mark Hayes
@@ -35,9 +33,7 @@ import com.sleepycat.je.SecondaryDatabase;
 public class TupleSerialFactoryTest extends TestCase
     implements TransactionWorker {
 
-    public static void main(String[] args)
-        throws Exception {
-
+    public static void main(String[] args) {
         junit.framework.TestResult tr =
             junit.textui.TestRunner.run(suite());
         if (tr.errorCount() > 0 ||
@@ -48,9 +44,7 @@ public class TupleSerialFactoryTest extends TestCase
         }
     }
 
-    public static Test suite()
-        throws Exception {
-
+    public static Test suite() {
         TestSuite suite = new TestSuite();
         for (int i = 0; i < TestEnv.ALL.length; i += 1) {
             for (int sorted = 0; sorted < 2; sorted += 1) {
@@ -70,7 +64,7 @@ public class TupleSerialFactoryTest extends TestCase
     private Database store2;
     private SecondaryDatabase index1;
     private SecondaryDatabase index2;
-    private boolean isSorted;
+    private final boolean isSorted;
     private Map storeMap1;
     private Map storeMap2;
     private Map indexMap1;
@@ -88,16 +82,18 @@ public class TupleSerialFactoryTest extends TestCase
         setName(name);
     }
 
+    @Override
     public void setUp()
         throws Exception {
 
-        DbTestUtil.printTestName(getName());
+        SharedTestUtils.printTestName(getName());
         env = testEnv.open(getName());
         runner = new TransactionRunner(env);
 
         createDatabase();
     }
 
+    @Override
     public void tearDown() {
 
         try {
@@ -139,15 +135,14 @@ public class TupleSerialFactoryTest extends TestCase
         }
     }
 
+    @Override
     public void runTest()
         throws Exception {
 
         runner.run(this);
     }
 
-    public void doWork()
-        throws Exception {
-
+    public void doWork() {
         createViews();
         writeAndRead();
     }
@@ -171,8 +166,9 @@ public class TupleSerialFactoryTest extends TestCase
         DatabaseConfig config = new DatabaseConfig();
         config.setTransactional(testEnv.isTxnMode());
         config.setAllowCreate(true);
+        DbCompat.setTypeBtree(config);
 
-        return DbCompat.openDatabase(env, null, file, null, config);
+        return DbCompat.testOpenDatabase(env, null, file, null, config);
     }
 
     private SecondaryDatabase openSecondaryDb(TupleSerialFactory factory,
@@ -185,6 +181,7 @@ public class TupleSerialFactoryTest extends TestCase
         SecondaryConfig secConfig = new SecondaryConfig();
         secConfig.setTransactional(testEnv.isTxnMode());
         secConfig.setAllowCreate(true);
+        DbCompat.setTypeBtree(secConfig);
         secConfig.setKeyCreator(factory.getKeyCreator(MarshalledObject.class,
                                                       keyName));
         if (foreignStore != null) {
@@ -193,14 +190,11 @@ public class TupleSerialFactoryTest extends TestCase
                     ForeignKeyDeleteAction.CASCADE);
         }
 
-        return DbCompat.openSecondaryDatabase(env, null,
-                                              file, null,
-                                              primary, secConfig);
+        return DbCompat.testOpenSecondaryDatabase
+            (env, null, file, null, primary, secConfig);
     }
 
-    private void createViews()
-        throws Exception {
-
+    private void createViews() {
         if (isSorted) {
             storeMap1 = factory.newSortedMap(store1, String.class,
                                              MarshalledObject.class, true);
@@ -222,9 +216,7 @@ public class TupleSerialFactoryTest extends TestCase
         }
     }
 
-    private void writeAndRead()
-        throws Exception {
-
+    private void writeAndRead() {
         MarshalledObject o1 = new MarshalledObject("data1", "pk1", "ik1", "");
         assertNull(storeMap1.put(null, o1));
 
