@@ -1,16 +1,13 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: SR10597Test.java,v 1.5 2006/01/03 21:56:12 bostic Exp $
  */
 
 package com.sleepycat.je.cleaner;
 
 import java.io.File;
-import java.io.IOException;
 
 import junit.framework.TestCase;
 
@@ -45,16 +42,14 @@ public class SR10597Test extends TestCase {
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
-    public void setUp()
-        throws IOException, DatabaseException {
-
+    @Override
+    public void setUp() {
         TestUtils.removeLogFiles("Setup", envHome, false);
         TestUtils.removeFiles("Setup", envHome, FileManager.DEL_SUFFIX);
     }
 
-    public void tearDown()
-        throws IOException, DatabaseException {
-
+    @Override
+    public void tearDown() {
         try {
             if (env != null) {
                 env.close();
@@ -62,7 +57,7 @@ public class SR10597Test extends TestCase {
         } catch (Throwable e) {
             System.out.println("tearDown: " + e);
         }
-                
+
         try {
             TestUtils.removeLogFiles("tearDown", envHome, true);
             TestUtils.removeFiles("tearDown", envHome, FileManager.DEL_SUFFIX);
@@ -82,7 +77,7 @@ public class SR10597Test extends TestCase {
         throws DatabaseException {
 
         EnvironmentConfig config = TestUtils.initEnvConfig();
-	DbInternal.disableParameterValidation(config);
+        DbInternal.disableParameterValidation(config);
         config.setAllowCreate(true);
         /* Do not run the daemons. */
         config.setConfigParam
@@ -90,7 +85,7 @@ public class SR10597Test extends TestCase {
         config.setConfigParam
             (EnvironmentParams.ENV_RUN_EVICTOR.getName(), "false");
         config.setConfigParam
-	    (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
+            (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
         config.setConfigParam
             (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
         /* Use a small log file size to make cleaning more frequent. */
