@@ -1,16 +1,15 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: UtilizationTest.java,v 1.17 2006/01/03 21:56:13 bostic Exp $
  */
 
 package com.sleepycat.je.cleaner;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.ByteBuffer;
 import java.util.Enumeration;
 
 import junit.framework.Test;
@@ -30,9 +29,20 @@ import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
+import com.sleepycat.je.dbi.DatabaseImpl;
+import com.sleepycat.je.dbi.EnvironmentImpl;
+import com.sleepycat.je.log.ChecksumException;
 import com.sleepycat.je.log.FileManager;
+import com.sleepycat.je.log.LogEntryHeader;
+import com.sleepycat.je.log.LogEntryType;
+import com.sleepycat.je.log.LogManager;
+import com.sleepycat.je.log.LogSource;
 import com.sleepycat.je.util.TestUtils;
+import com.sleepycat.je.utilint.DbLsn;
 
+/**
+ * Test utilization counting of LNs.
+ */
 public class UtilizationTest extends TestCase {
 
     private static final String DB_NAME = "foo";
@@ -40,9 +50,23 @@ public class UtilizationTest extends TestCase {
     private static final String OP_NONE = "op-none";
     private static final String OP_CHECKPOINT = "op-checkpoint";
     private static final String OP_RECOVER = "op-recover";
+    //private static final String[] OPERATIONS = { OP_NONE, };
+    //*
     private static final String[] OPERATIONS = { OP_NONE,
                                                  OP_CHECKPOINT,
+                                                 OP_RECOVER,
                                                  OP_RECOVER };
+    //*/
+
+    /*
+     * Set fetchObsoleteSize=true only for the second OP_RECOVER test.
+     * We check that OP_RECOVER works with without fetching, but with fetching
+     * we check that all LN sizes are counted.
+     */
+    private static final boolean[] FETCH_OBSOLETE_SIZE = { false,
+                                                           false,
+                                                           false,
+                                                           true };
 
     private static final CheckpointConfig forceConfig = new CheckpointConfig();
     static {
@@ -51,12 +75,16 @@ public class UtilizationTest extends TestCase {
 
     private File envHome;
     private Environment env;
+    private EnvironmentImpl envImpl;
     private Database db;
+    private DatabaseImpl dbImpl;
     private boolean dups = false;
     private DatabaseEntry keyEntry = new DatabaseEntry();
     private DatabaseEntry dataEntry = new DatabaseEntry();
     private String operation;
     private long lastFileSeen;
+    private boolean fetchObsoleteSize;
+    private boolean truncateOrRemoveDone;
 
     public static Test suite() {
         TestSuite allTests = new TestSuite();
@@ -65,7 +93,7 @@ public class UtilizationTest extends TestCase {
             Enumeration e = suite.tests();
             while (e.hasMoreElements()) {
                 UtilizationTest test = (UtilizationTest) e.nextElement();
-                test.init(OPERATIONS[i]);
+                test.init(OPERATIONS[i], FETCH_OBSOLETE_SIZE[i]);
                 allTests.addTest(test);
             }
         }
@@ -76,22 +104,23 @@ public class UtilizationTest extends TestCase {
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
-    private void init(String operation) {
+    private void init(String operation, boolean fetchObsoleteSize) {
         this.operation = operation;
+        this.fetchObsoleteSize = fetchObsoleteSize;
     }
 
-    public void setUp()
-        throws IOException, DatabaseException {
-
+    @Override
+    public void setUp() {
         TestUtils.removeLogFiles("Setup", envHome, false);
         TestUtils.removeFiles("Setup", envHome, FileManager.DEL_SUFFIX);
     }
 
-    public void tearDown()
-        throws IOException, DatabaseException {
-        
+    @Override
+    public void tearDown() {
         /* Set test name for reporting; cannot be done in the ctor or setUp. */
-        setName(operation + ':' + getName());
+        setName(operation +
+                (fetchObsoleteSize ? "-fetch" : "") +
+                ':' + getName());
 
         try {
             if (env != null) {
@@ -100,9 +129,9 @@ public class UtilizationTest extends TestCase {
         } catch (Throwable e) {
             System.out.println("tearDown: " + e);
         }
-                
+
         try {
-            /*
+            //*
             TestUtils.removeLogFiles("tearDown", envHome, true);
             TestUtils.removeFiles("tearDown", envHome, FileManager.DEL_SUFFIX);
             //*/
@@ -111,7 +140,9 @@ public class UtilizationTest extends TestCase {
         }
 
         db = null;
+        dbImpl = null;
         env = null;
+        envImpl = null;
         envHome = null;
         keyEntry = null;
         dataEntry = null;
@@ -124,7 +155,7 @@ public class UtilizationTest extends TestCase {
         throws DatabaseException {
 
         EnvironmentConfig config = TestUtils.initEnvConfig();
-	DbInternal.disableParameterValidation(config);
+        DbInternal.disableParameterValidation(config);
         config.setTransactional(true);
         config.setTxnNoSync(true);
         config.setAllowCreate(true);
@@ -134,21 +165,25 @@ public class UtilizationTest extends TestCase {
         config.setConfigParam
             (EnvironmentParams.ENV_RUN_EVICTOR.getName(), "false");
         config.setConfigParam
-	    (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
+            (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
         config.setConfigParam
             (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
         /* Use a tiny log file size to write one LN per file. */
         config.setConfigParam(EnvironmentParams.LOG_FILE_MAX.getName(),
                               Integer.toString(64));
-        /* Don't use NIO direct buffers or we run out of memory. */
-        config.setConfigParam
-            (EnvironmentParams.LOG_DIRECT_NIO.getName(), "false");
+
+        /* Obsolete LN size counting is optional per test. */
+        if (fetchObsoleteSize) {
+            config.setConfigParam
+                (EnvironmentParams.CLEANER_FETCH_OBSOLETE_SIZE.getName(),
+                 "true");
+        }
+
         env = new Environment(envHome, config);
+        envImpl = DbInternal.getEnvironmentImpl(env);
 
         /* Speed up test that uses lots of very small files. */
-        DbInternal.envGetEnvironmentImpl(env).
-                   getFileManager().
-                   setSyncAtFileEnd(false);
+        envImpl.getFileManager().setSyncAtFileEnd(false);
 
         openDb();
     }
@@ -164,6 +199,7 @@ public class UtilizationTest extends TestCase {
         dbConfig.setAllowCreate(true);
         dbConfig.setSortedDuplicates(dups);
         db = env.openDatabase(null, DB_NAME, dbConfig);
+        dbImpl = DbInternal.getDatabaseImpl(db);
     }
 
     /**
@@ -172,12 +208,23 @@ public class UtilizationTest extends TestCase {
     private void closeEnv(boolean doCheckpoint)
         throws DatabaseException {
 
+        /*
+         * We pass expectAccurateDbUtilization as false when
+         * truncateOrRemoveDone, because the database utilization info for that
+         * database is now gone.
+         */
+        VerifyUtils.verifyUtilization
+            (envImpl,
+             true, // expectAccurateObsoleteLNCount
+             expectAccurateObsoleteLNSize(),
+             !truncateOrRemoveDone); // expectAccurateDbUtilization
+
         if (db != null) {
             db.close();
             db = null;
         }
         if (env != null) {
-            DbInternal.envGetEnvironmentImpl(env).close(doCheckpoint);
+            envImpl.close(doCheckpoint);
             env = null;
         }
     }
@@ -345,9 +392,12 @@ public class UtilizationTest extends TestCase {
         performRecoveryOperation();
 
         /* Expect that LN is not obsolete. */
-        FileSummary summary = getSummary(file0);
-        assertEquals(1, summary.totalLNCount);
-        assertEquals(0, summary.obsoleteLNCount);
+        FileSummary fileSummary = getFileSummary(file0);
+        assertEquals(1, fileSummary.totalLNCount);
+        assertEquals(0, fileSummary.obsoleteLNCount);
+        DbFileSummary dbFileSummary = getDbFileSummary(file0);
+        assertEquals(1, dbFileSummary.totalLNCount);
+        assertEquals(0, dbFileSummary.obsoleteLNCount);
 
         closeEnv(true);
     }
@@ -362,9 +412,12 @@ public class UtilizationTest extends TestCase {
         performRecoveryOperation();
 
         /* Expect that LN is obsolete. */
-        FileSummary summary = getSummary(file0);
-        assertEquals(1, summary.totalLNCount);
-        assertEquals(1, summary.obsoleteLNCount);
+        FileSummary fileSummary = getFileSummary(file0);
+        assertEquals(1, fileSummary.totalLNCount);
+        assertEquals(1, fileSummary.obsoleteLNCount);
+        DbFileSummary dbFileSummary = getDbFileSummary(file0);
+        assertEquals(1, dbFileSummary.totalLNCount);
+        assertEquals(1, dbFileSummary.obsoleteLNCount);
 
         closeEnv(true);
     }
@@ -1064,7 +1117,6 @@ public class UtilizationTest extends TestCase {
     }
 
     /**
-     * @deprecated use of Database.truncate
      */
     private void truncateOrRemove(boolean truncate, boolean commit)
         throws DatabaseException {
@@ -1082,7 +1134,10 @@ public class UtilizationTest extends TestCase {
         /* Truncate. */
         txn = env.beginTransaction(null, null);
         if (truncate) {
-            int count = db.truncate(txn, true);
+            db.close();
+            db = null;
+            long count = env.truncateDatabase(txn, DB_NAME,
+                                              true /* returnCount */);
             assertEquals(3, count);
         } else {
             db.close();
@@ -1094,23 +1149,136 @@ public class UtilizationTest extends TestCase {
         } else {
             txn.abort();
         }
+        truncateOrRemoveDone = true;
         performRecoveryOperation();
 
-        expectObsolete(file0, commit);
-        expectObsolete(file1, commit);
-        expectObsolete(file2, commit);
+        /*
+         * Do not check DbFileSummary when we truncate/remove, since the old
+         * DatabaseImpl is gone.
+         */
+        expectObsolete(file0, commit, !commit /*checkDbFileSummary*/);
+        expectObsolete(file1, commit, !commit /*checkDbFileSummary*/);
+        expectObsolete(file2, commit, !commit /*checkDbFileSummary*/);
 
         closeEnv(true);
     }
 
+    /*
+     * The xxxForceTreeWalk tests set the DatabaseImpl
+     * forceTreeWalkForTruncateAndRemove field to true, which will force a walk
+     * of the tree to count utilization during truncate/remove, rather than
+     * using the per-database info.  This is used to test the "old technique"
+     * for counting utilization, which is now used only if the database was
+     * created prior to log version 6.
+     */
+
+    public void testTruncateForceTreeWalk()
+        throws Exception {
+
+        DatabaseImpl.forceTreeWalkForTruncateAndRemove = true;
+        try {
+            testTruncate();
+        } finally {
+            DatabaseImpl.forceTreeWalkForTruncateAndRemove = false;
+        }
+    }
+
+    public void testTruncateAbortForceTreeWalk()
+        throws Exception {
+
+        DatabaseImpl.forceTreeWalkForTruncateAndRemove = true;
+        try {
+            testTruncateAbort();
+        } finally {
+            DatabaseImpl.forceTreeWalkForTruncateAndRemove = false;
+        }
+    }
+
+    public void testRemoveForceTreeWalk()
+        throws Exception {
+
+        DatabaseImpl.forceTreeWalkForTruncateAndRemove = true;
+        try {
+            testRemove();
+        } finally {
+            DatabaseImpl.forceTreeWalkForTruncateAndRemove = false;
+        }
+    }
+
+    public void testRemoveAbortForceTreeWalk()
+        throws Exception {
+
+        DatabaseImpl.forceTreeWalkForTruncateAndRemove = true;
+        try {
+            testRemoveAbort();
+        } finally {
+            DatabaseImpl.forceTreeWalkForTruncateAndRemove = false;
+        }
+    }
+
     private void expectObsolete(long file, boolean obsolete)
         throws DatabaseException {
 
-        FileSummary summary = getSummary(file);
+        expectObsolete(file, obsolete, true /*checkDbFileSummary*/);
+    }
+
+    private void expectObsolete(long file,
+                                boolean obsolete,
+                                boolean checkDbFileSummary)
+        throws DatabaseException {
+
+        FileSummary fileSummary = getFileSummary(file);
         assertEquals("totalLNCount",
-                     1, summary.totalLNCount);
+                     1, fileSummary.totalLNCount);
         assertEquals("obsoleteLNCount",
-                     obsolete ? 1 : 0, summary.obsoleteLNCount);
+                     obsolete ? 1 : 0, fileSummary.obsoleteLNCount);
+
+        DbFileSummary dbFileSummary = getDbFileSummary(file);
+        if (checkDbFileSummary) {
+            assertEquals("db totalLNCount",
+                         1, dbFileSummary.totalLNCount);
+            assertEquals("db obsoleteLNCount",
+                         obsolete ? 1 : 0, dbFileSummary.obsoleteLNCount);
+        }
+
+        if (obsolete) {
+            if (expectAccurateObsoleteLNSize()) {
+                assertTrue(fileSummary.obsoleteLNSize > 0);
+                assertEquals(1, fileSummary.obsoleteLNSizeCounted);
+                if (checkDbFileSummary) {
+                    assertTrue(dbFileSummary.obsoleteLNSize > 0);
+                    assertEquals(1, dbFileSummary.obsoleteLNSizeCounted);
+                }
+            }
+            /* If we counted the size, make sure it is the actual LN size. */
+            if (fileSummary.obsoleteLNSize > 0) {
+                assertEquals(getLNSize(file), fileSummary.obsoleteLNSize);
+            }
+            if (checkDbFileSummary) {
+                if (dbFileSummary.obsoleteLNSize > 0) {
+                    assertEquals(getLNSize(file), dbFileSummary.obsoleteLNSize);
+                }
+                assertEquals(fileSummary.obsoleteLNSize > 0,
+                             dbFileSummary.obsoleteLNSize > 0);
+            }
+        } else {
+            assertEquals(0, fileSummary.obsoleteLNSize);
+            assertEquals(0, fileSummary.obsoleteLNSizeCounted);
+            if (checkDbFileSummary) {
+                assertEquals(0, dbFileSummary.obsoleteLNSize);
+                assertEquals(0, dbFileSummary.obsoleteLNSizeCounted);
+            }
+        }
+    }
+
+    /**
+     * If an LN is obsolete, expect the size to be counted unless we ran
+     * recovery and we did NOT configure fetchObsoleteSize=true.  In that
+     * case, the size may or may not be counted depending on how the redo
+     * or undo was processed during reocvery.
+     */
+    private boolean expectAccurateObsoleteLNSize() {
+        return fetchObsoleteSize || !OP_RECOVER.equals(operation);
     }
 
     private long doPut(int key, boolean commit)
@@ -1199,7 +1367,8 @@ public class UtilizationTest extends TestCase {
     }
 
     /**
-     * Checkpoint, recover, or do nothing.
+     * Checkpoint, recover, or do neither, depending on the configured
+     * operation for this test.  Always compress to count deleted LNs.
      */
     private void performRecoveryOperation()
         throws DatabaseException {
@@ -1226,9 +1395,7 @@ public class UtilizationTest extends TestCase {
      * Also check that the file number is greater than the last file returned,
      * to ensure that we're filling a file every time we write.
      */
-    private long getFile(Cursor cursor)
-        throws DatabaseException {
-
+    private long getFile(Cursor cursor) {
         long file = CleanerTestUtils.getLogFile(this, cursor);
         assert file > lastFileSeen;
         lastFileSeen = file;
@@ -1238,13 +1405,42 @@ public class UtilizationTest extends TestCase {
     /**
      * Returns the utilization summary for a given log file.
      */
-    private FileSummary getSummary(long file)
+    private FileSummary getFileSummary(long file) {
+        return envImpl.getUtilizationProfile()
+               .getFileSummaryMap(true)
+               .get(new Long(file));
+    }
+
+    /**
+     * Returns the per-database utilization summary for a given log file.
+     */
+    private DbFileSummary getDbFileSummary(long file) {
+        return dbImpl.getDbFileSummary
+            (new Long(file), false /*willModify*/);
+    }
+
+    /**
+     * Peek into the file to get the total size of the first entry past the
+     * file header, which is known to be the LN log entry.
+     */
+    private int getLNSize(long file)
         throws DatabaseException {
 
-        return (FileSummary)
-            DbInternal.envGetEnvironmentImpl(env)
-                      .getUtilizationProfile()
-                      .getFileSummaryMap(true)
-                      .get(new Long(file));
+        try {
+            long offset = FileManager.firstLogEntryOffset();
+            long lsn = DbLsn.makeLsn(file, offset);
+            LogManager lm = envImpl.getLogManager();
+            LogSource src = lm.getLogSource(lsn);
+            ByteBuffer buf = src.getBytes(offset);
+            LogEntryHeader header =
+                new LogEntryHeader(buf, LogEntryType.LOG_VERSION);
+            int size = header.getItemSize();
+            src.release();
+            return size + header.getSize();
+        } catch (IOException e) {
+            throw new RuntimeException(e);
+        } catch (ChecksumException e) {
+            throw new RuntimeException(e);
+        }
     }
 }
