@@ -1,10 +1,8 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: FSyncManagerTest.java,v 1.8 2006/01/03 21:56:20 bostic Exp $
  */
 
 package com.sleepycat.je.log;
@@ -13,7 +11,6 @@ import java.io.File;
 
 import junit.framework.TestCase;
 
-import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
@@ -25,27 +22,26 @@ import com.sleepycat.je.util.TestUtils;
 /**
  * Exercise the synchronization aspects of the sync manager.
  */
-public class FSyncManagerTest extends TestCase { 
-    private File envHome;
+public class FSyncManagerTest extends TestCase {
+    private final File envHome;
 
     public FSyncManagerTest() {
         super();
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
-    protected void setUp()
-        throws Exception {
+    @Override
+    protected void setUp() {
         /* Remove files to start with a clean slate. */
         TestUtils.removeLogFiles("Setup", envHome, false);
     }
 
-    protected void tearDown()
-        throws Exception {
-
+    @Override
+    protected void tearDown() {
         TestUtils.removeLogFiles("TearDown", envHome, false);
     }
 
-    public void testBasic() 
+    public void testBasic()
         throws Throwable{
         Environment env = null;
 
@@ -59,7 +55,7 @@ public class FSyncManagerTest extends TestCase {
             WaitVal waitVal = new WaitVal(0);
 
             FSyncManager syncManager =
-                new TestSyncManager(DbInternal.envGetEnvironmentImpl(env),
+                new TestSyncManager(DbInternal.getEnvironmentImpl(env),
                                     waitVal);
             JUnitThread t1 = new TestSyncThread(syncManager);
             JUnitThread t2 = new TestSyncThread(syncManager);
@@ -80,7 +76,7 @@ public class FSyncManagerTest extends TestCase {
             t1.join();
             t2.join();
             t3.join();
-            
+
             /*
              * All three threads ask for fsyncs.
              * 2 do fsyncs -- the initial leader, and the leader of the
@@ -101,20 +97,19 @@ public class FSyncManagerTest extends TestCase {
      * This way, we can manipulate grouping behavior.
      */
     class TestSyncManager extends FSyncManager {
-        private WaitVal waitVal;
-        TestSyncManager(EnvironmentImpl env, WaitVal waitVal)
-            throws DatabaseException {
+        private final WaitVal waitVal;
+        TestSyncManager(EnvironmentImpl env, WaitVal waitVal) {
             super(env);
             this.waitVal = waitVal;
         }
-        protected void executeFSync() 
-            throws DatabaseException {
+        @Override
+        protected void executeFSync() {
             try {
                 synchronized (waitVal) {
                     if (waitVal.value < 1) {
                         waitVal.wait();
                     }
-                } 
+                }
             } catch (InterruptedException e) {
                 // woken up.
             }
@@ -122,13 +117,14 @@ public class FSyncManagerTest extends TestCase {
     }
 
     class TestSyncThread extends JUnitThread {
-        private FSyncManager syncManager;
+        private final FSyncManager syncManager;
         TestSyncThread(FSyncManager syncManager) {
             super("syncThread");
             this.syncManager = syncManager;
         }
-        
-        public void testBody() 
+
+        @Override
+        public void testBody()
             throws Throwable {
             syncManager.fsync();
         }
