@@ -1,11 +1,9 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: FileManagerTestUtils.java,v 1.1 2004/05/17 02:59:47 linda Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
+ *
+ */
 
 package com.sleepycat.je.log;
 
@@ -16,17 +14,15 @@ import com.sleepycat.je.dbi.EnvironmentImpl;
 
 public class FileManagerTestUtils {
     public static void createLogFile(FileManager fileManager,
-    	                             EnvironmentImpl envImpl,
-    	                             long logFileSize)
+                                         EnvironmentImpl envImpl,
+                                         long logFileSize)
         throws DatabaseException, IOException {
 
         LogBuffer logBuffer = new LogBuffer(50, envImpl);
         logBuffer.getDataBuffer().flip();
         fileManager.bumpLsn(logFileSize - FileManager.firstLogEntryOffset());
         logBuffer.registerLsn(fileManager.getLastUsedLsn());
-        fileManager.writeLogBuffer(logBuffer);
+        fileManager.writeLogBuffer(logBuffer, true);
         fileManager.syncLogEndAndFinishFile();
-
     }
 }
-
