@@ -1,17 +1,17 @@
 /*-
-* See the file LICENSE for redistribution information.
-*
-* Copyright (c) 2002-2004
-*      Sleepycat Software.  All rights reserved.
-*
-* $Id: RecoveryAbortTest.java,v 1.43 2004/07/30 15:51:09 cwl Exp $
-*/
+ * See the file LICENSE for redistribution information.
+ *
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
+ *
+ */
 
 package com.sleepycat.je.recovery;
 
-import java.util.Hashtable;
-import java.util.Properties;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.Set;
 
+import com.sleepycat.je.Cursor;
 import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseEntry;
@@ -19,6 +19,7 @@ import com.sleepycat.je.DatabaseException;
 import com.sleepycat.je.DbInternal;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
+import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.EnvironmentImpl;
@@ -28,22 +29,23 @@ public class RecoveryAbortTest extends RecoveryTestBase {
     private static final boolean DEBUG = false;
 
     public RecoveryAbortTest() {
-	super(true);
+        super(true);
     }
 
     /**
      * Insert data into several dbs, then abort.
      */
     public void testBasic()
-	throws Throwable {
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, true, NUM_DBS);
+        createEnvAndDbs(1 << 20, true, NUM_DBS);
         int numRecs = NUM_RECS * 3;
 
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert all the data. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs - 1, expectedData, 1, false, NUM_DBS);
@@ -61,43 +63,45 @@ public class RecoveryAbortTest extends RecoveryTestBase {
      * Test insert/abort with no duplicates.
      */
     public void testInserts()
-	throws Throwable {
+        throws Throwable {
+
+        createEnvAndDbs(1 << 20, true, NUM_DBS);
+        EnvironmentImpl realEnv = DbInternal.getEnvironmentImpl(env);
 
-	createEnvAndDbs(1 << 20, true, NUM_DBS);
-        EnvironmentImpl realEnv = DbInternal.envGetEnvironmentImpl(env);
-        
         int N = NUM_RECS;
-        
+
         if (DEBUG) {
             System.out.println("<dump>");
         }
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert 0 - N and commit. */
             Transaction txn = env.beginTransaction(null, null);
-            insertData(txn, 0, N-1, expectedData, 1, true, NUM_DBS);
+            insertData(txn, 0, N - 1, expectedData, 1, true, NUM_DBS);
             txn.commit();
             verifyData(expectedData, false, NUM_DBS);
-            
+
             /* Insert N - 3N and abort. */
             txn = env.beginTransaction(null, null);
-            insertData(txn, N, (3*N)-1, expectedData, 1, false, NUM_DBS);
+            insertData(txn, N, (3 * N) - 1, expectedData, 1, false, NUM_DBS);
             txn.abort();
             verifyData(expectedData, false, NUM_DBS);
 
-            /* 
-	     * Wait for the incompressor queue to be processed, so we
-             * force the recovery to run w/IN delete replays.
-	     */
-            while (realEnv.getInCompressorQueueSize() > 0) {
+            /*
+             * Wait for the incompressor queue to be processed, so we force the
+             * recovery to run w/IN delete replays.
+             */
+            while (realEnv.getINCompressorQueueSize() > 0) {
                 Thread.sleep(10000);
             }
 
             /* Insert 2N - 4N and commit. */
             txn = env.beginTransaction(null, null);
-            insertData(txn, (2*N), (4*N)-1, expectedData, 1, true, NUM_DBS);
+            insertData(txn, (2 * N), (4 * N) - 1, expectedData, 1, true,
+                       NUM_DBS);
             txn.commit();
             verifyData(expectedData, false, NUM_DBS);
 
@@ -115,18 +119,19 @@ public class RecoveryAbortTest extends RecoveryTestBase {
         }
     }
 
-    public void testMix() 
-	throws Throwable {
+    public void testMix()
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, true, NUM_DBS);
+        createEnvAndDbs(1 << 20, true, NUM_DBS);
 
         int numRecs = NUM_RECS;
-        int numDups = 10;        
-                
+        int numDups = 10;
+
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert data without duplicates. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs, expectedData, 1, true, NUM_DBS);
@@ -145,7 +150,7 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             txn = env.beginTransaction(null, null);
             deleteData(txn, expectedData, false, true, NUM_DBS);
             txn.commit();
-            
+
             /* Modify some and abort. */
             txn = env.beginTransaction(null, null);
             modifyData(txn, numRecs, expectedData, 3, false, NUM_DBS);
@@ -159,11 +164,12 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
-            TestUtils.validateMemoryUsage
-                           (DbInternal.envGetEnvironmentImpl(env),
+            TestUtils.validateNodeMemUsage
+                           (DbInternal.getEnvironmentImpl(env),
                             false);
             closeEnv();
             recoverAndVerify(expectedData, NUM_DBS);
@@ -174,6 +180,97 @@ public class RecoveryAbortTest extends RecoveryTestBase {
         }
     }
 
+    public void testSR13726()
+        throws Throwable {
+
+        int numDbs = 1;
+
+        createEnvAndDbs(1 << 20, true, numDbs);
+
+        try {
+            /*
+             * Insert data without duplicates, commit. This gets us a
+             * DupCountLN.
+             */
+            Transaction txn = env.beginTransaction(null, null);
+            Cursor c = dbs[0].openCursor(txn, null);
+            DatabaseEntry key = new DatabaseEntry();
+            DatabaseEntry data = new DatabaseEntry();
+            byte[] keyData = TestUtils.getTestArray(0);
+            byte[] dataData = TestUtils.byteArrayCopy(keyData);
+            key.setData(keyData);
+            data.setData(dataData);
+            for (int i = 0; i < 3; i++) {
+                data.setData(TestUtils.getTestArray(i));
+                assertEquals("insert some dups",
+                             c.put(key, data),
+                             OperationStatus.SUCCESS);
+            }
+            c.close();
+            txn.commit();
+
+            /* This gets us a DelDupLN in the slot in the BIN. */
+            txn = env.beginTransaction(null, null);
+            assertEquals("delete initial dups",
+                         dbs[0].delete(txn, key),
+                         OperationStatus.SUCCESS);
+            txn.commit();
+
+            /* This gets the dup tree cleaned up. */
+            env.compress();
+
+            /* Gets the BIN written out with knownDeleted=true. */
+            closeEnv();
+            recoverOnly(numDbs);
+            createDbs(null, numDbs);
+
+            /*
+             * Tree now has a BIN referring to a DelDupLN.  Add duplicates,
+             * and abort.
+             */
+            txn = env.beginTransaction(null, null);
+            c = dbs[0].openCursor(txn, null);
+            for (int i = 0; i < 3; i++) {
+                data.setData(TestUtils.getTestArray(i));
+                assertEquals("insert later dups",
+                             c.put(key, data),
+                             OperationStatus.SUCCESS);
+            }
+            c.close();
+            txn.abort();
+
+            /*
+             * Now add duplicates again and commit.
+             */
+            txn = env.beginTransaction(null, null);
+            c = dbs[0].openCursor(txn, null);
+            for (int i = 0; i < 3; i++) {
+                data.setData(TestUtils.getTestArray(i));
+                assertEquals("insert later dups",
+                             c.put(key, data),
+                             OperationStatus.SUCCESS);
+            }
+            c.close();
+            txn.commit();
+
+            txn = env.beginTransaction(null, null);
+            c = dbs[0].openCursor(txn, null);
+            int count = 0;
+            while (c.getNext(key, data, null) == OperationStatus.SUCCESS) {
+                count++;
+            }
+            c.getSearchKey(key, data, null);
+            assertEquals("scanned count == count()", count, c.count());
+            c.close();
+            txn.commit();
+            closeEnv();
+        } catch (Throwable t) {
+            // print stacktrace before trying to clean up files
+            t.printStackTrace();
+            throw t;
+        }
+    }
+
     /*
      * Test the sequence where we have an existing record in the
      * database; then in a separate transaction we delete that data
@@ -188,16 +285,17 @@ public class RecoveryAbortTest extends RecoveryTestBase {
      * prevented by a lock being taken by the compressor on the LN,
      * even if the LN is "knownDeleted". [#9465]
      */
-    public void testSR9465Part1() 
-	throws Throwable {
+    public void testSR9465Part1()
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, true, NUM_DBS);
+        createEnvAndDbs(1 << 20, true, NUM_DBS);
         int numRecs = NUM_RECS;
-        
+
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert data without duplicates. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs, expectedData, 1, true, NUM_DBS);
@@ -210,14 +308,15 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             txn.abort();
 
             txn = env.beginTransaction(null, null);
-	    verifyData(expectedData, NUM_DBS);
+            verifyData(expectedData, NUM_DBS);
             txn.commit();
 
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
 
             closeEnv();
@@ -229,16 +328,17 @@ public class RecoveryAbortTest extends RecoveryTestBase {
         }
     }
 
-    public void testSR9465Part2() 
-	throws Throwable {
+    public void testSR9465Part2()
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, true, NUM_DBS);
+        createEnvAndDbs(1 << 20, true, NUM_DBS);
         int numRecs = NUM_RECS;
-        
+
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert data without duplicates. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs, expectedData, 1, true, NUM_DBS);
@@ -254,19 +354,21 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
 
             txn = env.beginTransaction(null, null);
-	    verifyData(expectedData, NUM_DBS);
+            verifyData(expectedData, NUM_DBS);
             txn.commit();
 
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
 
             closeEnv();
@@ -278,36 +380,37 @@ public class RecoveryAbortTest extends RecoveryTestBase {
         }
     }
 
-    public void testSR9752Part1() 
-	throws Throwable {
+    public void testSR9752Part1()
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, false, NUM_DBS);
+        createEnvAndDbs(1 << 20, false, NUM_DBS);
         int numRecs = NUM_RECS;
-        
+
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert data without duplicates. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs, expectedData, 1, true, NUM_DBS);
             txn.commit();
 
-	    /*
-	     * txn1 just puts a piece of data out to a database that won't
-	     * be seen by deleteData or insertData.  The idea is to hold
-	     * the transaction open across the env.sync() so that firstActive
-	     * comes before ckptStart.
-	     */
+            /*
+             * txn1 just puts a piece of data out to a database that won't
+             * be seen by deleteData or insertData.  The idea is to hold
+             * the transaction open across the env.sync() so that firstActive
+             * comes before ckptStart.
+             */
             Transaction txn1 = env.beginTransaction(null, null);
-	    DatabaseEntry key = new DatabaseEntry(new byte[] { 1, 2, 3, 4 });
-	    DatabaseEntry data = new DatabaseEntry(new byte[] { 4, 3, 2, 1 });
-	    DatabaseConfig dbConfig = new DatabaseConfig();
-	    dbConfig.setAllowCreate(true);
-	    dbConfig.setSortedDuplicates(false);
-	    dbConfig.setTransactional(true);
+            DatabaseEntry key = new DatabaseEntry(new byte[] { 1, 2, 3, 4 });
+            DatabaseEntry data = new DatabaseEntry(new byte[] { 4, 3, 2, 1 });
+            DatabaseConfig dbConfig = new DatabaseConfig();
+            dbConfig.setAllowCreate(true);
+            dbConfig.setSortedDuplicates(false);
+            dbConfig.setTransactional(true);
             Database otherDb = env.openDatabase(txn1, "extradb", dbConfig);
-	    otherDb.put(txn1, key, data);
+            otherDb.put(txn1, key, data);
 
             /* Delete all and abort. */
             txn = env.beginTransaction(null, null);
@@ -319,17 +422,18 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             deleteData(txn, expectedData, false, true, NUM_DBS);
             txn.commit();
 
-	    env.sync(); /* env.checkpoint does not seem to be sufficient. */
+            env.sync(); /* env.checkpoint does not seem to be sufficient. */
             txn1.commit();
-	    otherDb.close();
+            otherDb.close();
 
-	    closeEnv();
+            closeEnv();
 
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
 
             recoverAndVerify(expectedData, NUM_DBS);
@@ -341,36 +445,37 @@ public class RecoveryAbortTest extends RecoveryTestBase {
     }
 
     public void testSR9752Part2()
-	throws Throwable {
+        throws Throwable {
 
-	createEnvAndDbs(1 << 20, false, NUM_DBS);
-	DbInternal.envGetEnvironmentImpl(env).shutdownCleaner();
+        createEnvAndDbs(1 << 20, false, NUM_DBS);
+        DbInternal.getEnvironmentImpl(env).shutdownCleaner();
         int numRecs = NUM_RECS;
-        
+
         try {
             /* Set up an repository of expected data. */
-            Hashtable expectedData = new Hashtable();
-            
+            Map<TestData, Set<TestData>> expectedData = 
+                new HashMap<TestData, Set<TestData>>();
+
             /* Insert data without duplicates. */
             Transaction txn = env.beginTransaction(null, null);
             insertData(txn, 0, numRecs, expectedData, 1, true, NUM_DBS);
             txn.commit();
 
-	    /*
-	     * txn1 just puts a piece of data out to a database that won't
-	     * be seen by deleteData or insertData.  The idea is to hold
-	     * the transaction open across the env.sync() so that firstActive
-	     * comes before ckptStart.
-	     */
+            /*
+             * txn1 just puts a piece of data out to a database that won't
+             * be seen by deleteData or insertData.  The idea is to hold
+             * the transaction open across the env.sync() so that firstActive
+             * comes before ckptStart.
+             */
             Transaction txn1 = env.beginTransaction(null, null);
-	    DatabaseEntry key = new DatabaseEntry(new byte[] { 1, 2, 3, 4 });
-	    DatabaseEntry data = new DatabaseEntry(new byte[] { 4, 3, 2, 1 });
-	    DatabaseConfig dbConfig = new DatabaseConfig();
-	    dbConfig.setAllowCreate(true);
-	    dbConfig.setSortedDuplicates(false);
-	    dbConfig.setTransactional(true);
+            DatabaseEntry key = new DatabaseEntry(new byte[] { 1, 2, 3, 4 });
+            DatabaseEntry data = new DatabaseEntry(new byte[] { 4, 3, 2, 1 });
+            DatabaseConfig dbConfig = new DatabaseConfig();
+            dbConfig.setAllowCreate(true);
+            dbConfig.setSortedDuplicates(false);
+            dbConfig.setTransactional(true);
             Database otherDb = env.openDatabase(txn1, "extradb", dbConfig);
-	    otherDb.put(txn1, key, data);
+            otherDb.put(txn1, key, data);
 
             /* Delete all and abort. */
             txn = env.beginTransaction(null, null);
@@ -382,17 +487,18 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             deleteData(txn, expectedData, false, true, NUM_DBS);
             txn.commit();
 
-	    env.sync(); /* env.checkpoint does not seem to be sufficient. */
+            env.sync(); /* env.checkpoint does not seem to be sufficient. */
             txn1.commit();
-	    otherDb.close();
+            otherDb.close();
 
-	    closeEnv();
+            closeEnv();
 
             if (DEBUG) {
                 dumpData(NUM_DBS);
                 dumpExpected(expectedData);
-                com.sleepycat.je.tree.Key.DUMP_BINARY = true;
-                DbInternal.dbGetDatabaseImpl(dbs[0]).getTree().dump();
+                com.sleepycat.je.tree.Key.DUMP_TYPE =
+                    com.sleepycat.je.tree.Key.DumpType.BINARY;
+                DbInternal.getDatabaseImpl(dbs[0]).getTree().dump();
             }
 
             recoverAndVerify(expectedData, NUM_DBS);
@@ -406,8 +512,8 @@ public class RecoveryAbortTest extends RecoveryTestBase {
     /**
      * Insert dbs, commit some, abort some. To do: add db remove, rename.
      */
-    public void testDbCreateRemove() 
-	throws Throwable {
+    public void testDbCreateRemove()
+        throws Throwable {
 
         createEnv(1 << 20, true);
         int N1 = 10;
@@ -427,10 +533,10 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             dbConfig.setTransactional(true);
             dbConfig.setAllowCreate(true);
             for (int i = 0; i < N2; i++) {
-                Database db = env.openDatabase(txn, dbName1 + i, dbConfig);
+                env.openDatabase(txn, dbName1 + i, dbConfig);
             }
             txn.abort();
-            
+
             /* All dbs should not exist */
             checkForNoDb(dbName1, 0, N2);
 
@@ -452,11 +558,11 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             /* Close and recover */
             env.close();
 
-            EnvironmentConfig envConfig = new EnvironmentConfig();
-	    envConfig.setConfigParam
-		(EnvironmentParams.NODE_MAX.getName(), "6");
-	    envConfig.setConfigParam(EnvironmentParams.MAX_MEMORY.getName(),
-				     new Long(1 << 24).toString());
+            EnvironmentConfig envConfig = TestUtils.initEnvConfig();
+            envConfig.setConfigParam
+                (EnvironmentParams.NODE_MAX.getName(), "6");
+            envConfig.setConfigParam(EnvironmentParams.MAX_MEMORY.getName(),
+                                     new Long(1 << 24).toString());
             envConfig.setTransactional(true);
             envConfig.setTxnNoSync(Boolean.getBoolean(TestUtils.NO_SYNC));
             env = new Environment(envHome, envConfig);
@@ -507,12 +613,12 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             checkForDb(dbName1, N1, N3);
             checkForNoDb(dbName1, N3, N4);
             checkForDb(dbName1, N4, N5);
-            
+
             /* Rename some dbs, abort */
             txn = env.beginTransaction(null, null);
             for (int i = N1; i < N3; i++) {
                 env.renameDatabase
-		    (txn, dbName1+i, dbName2+i);
+                    (txn, dbName1+i, dbName2+i);
             }
             txn.abort();
 
@@ -520,7 +626,7 @@ public class RecoveryAbortTest extends RecoveryTestBase {
             txn = env.beginTransaction(null, null);
             for (int i = N2; i < N3; i++) {
                 env.renameDatabase
-		    (txn, dbName1+i, dbName2+i);
+                    (txn, dbName1+i, dbName2+i);
             }
             txn.commit();
 
@@ -546,14 +652,11 @@ public class RecoveryAbortTest extends RecoveryTestBase {
     /**
      * Fail if any db from start - (end -1) exists
      */
-    private void checkForNoDb(String dbName, int start, int end) 
-        throws DatabaseException {
-        	
+    private void checkForNoDb(String dbName, int start, int end) {
         /* Dbs start - end -1  shouldn't exist */
-        Properties emptyProps = new Properties();
         for (int i = start; i < end; i++) {
             try {
-                Database checkDb = env.openDatabase(null, dbName + i, null);
+                env.openDatabase(null, dbName + i, null);
                 fail(DB_NAME + i + " shouldn't exist");
             } catch (DatabaseException e) {
             }
@@ -563,10 +666,7 @@ public class RecoveryAbortTest extends RecoveryTestBase {
     /**
      * Fail if any db from start - (end -1) doesn't exist
      */
-    private void checkForDb(String dbName, int start, int end) 
-        throws DatabaseException {
-        Properties emptyProps = new Properties();
-
+    private void checkForDb(String dbName, int start, int end) {
         /* Dbs start - end -1  should exist. */
         for (int i = start; i < end; i++) {
             try {
