@@ -1,19 +1,16 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: PhantomRestartTest.java,v 1.9 2006/01/03 21:56:29 bostic Exp $
  */
 
 package com.sleepycat.je.test;
 
 import java.io.File;
-import java.io.IOException;
+import java.lang.reflect.Constructor;
 
 import junit.framework.Test;
-import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
 import com.sleepycat.bind.tuple.IntegerBinding;
@@ -22,22 +19,24 @@ import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DatabaseException;
-import com.sleepycat.je.DeadlockException;
+import com.sleepycat.je.Durability;
 import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
-import com.sleepycat.je.LockStats;
+import com.sleepycat.je.EnvironmentStats;
+import com.sleepycat.je.LockConflictException;
 import com.sleepycat.je.OperationStatus;
 import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.junit.JUnitThread;
 import com.sleepycat.je.log.FileManager;
+import com.sleepycat.je.util.DualTestCase;
 import com.sleepycat.je.util.TestUtils;
 
 /**
  * Tests read operation restarts that are the by product of phantom prevention
  * (range locking) added in SR [#10477].
  */
-public class PhantomRestartTest extends TestCase {
+public class PhantomRestartTest extends DualTestCase {
 
     /*
      * Spec Parameters: Oper name, InsertKey1, InsertKey2, Oper instance
@@ -181,7 +180,7 @@ public class PhantomRestartTest extends TestCase {
         }
 
         void checkStatus(OperationStatus expected) {
-            TestCase.assertEquals(expected, status);
+            assertEquals(expected, status);
         }
 
         void setKey(int val) {
@@ -195,11 +194,11 @@ public class PhantomRestartTest extends TestCase {
 
         void checkKey(int expected) {
             if (dups) {
-                TestCase.assertEquals(100, IntegerBinding.entryToInt(key));
-                TestCase.assertEquals
+                assertEquals(100, IntegerBinding.entryToInt(key));
+                assertEquals
                     (expected, IntegerBinding.entryToInt(data));
             } else {
-                TestCase.assertEquals
+                assertEquals
                     (expected, IntegerBinding.entryToInt(key));
             }
         }
@@ -208,7 +207,7 @@ public class PhantomRestartTest extends TestCase {
             throws DatabaseException;
     }
 
-    private static class Spec {
+    protected static class Spec {
 
         String name;
         int insertKey1;
@@ -223,14 +222,21 @@ public class PhantomRestartTest extends TestCase {
         }
     }
 
+    protected static Class<?> testClass = PhantomRestartTest.class;
+
     public static Test suite()
         throws Exception {
 
         TestSuite suite = new TestSuite();
+
+        Constructor constructor = testClass.getConstructor(
+                new Class[]{Spec.class, java.lang.Boolean.class});
+
         for (int i = 0; i < SPECS.length; i += 1) {
             for (int j = 0; j < 2; j += 1) {
                 boolean dups = (j != 0);
-                suite.addTest(new PhantomRestartTest(SPECS[i], dups));
+                suite.addTest(
+                       (DualTestCase) constructor.newInstance(SPECS[i], dups));
             }
         }
         return suite;
@@ -246,36 +252,31 @@ public class PhantomRestartTest extends TestCase {
     private boolean dups;
     private Spec spec;
 
-    public PhantomRestartTest(Spec spec, boolean dups) {
+    public PhantomRestartTest(Spec spec, Boolean dups) {
         super(spec.name + (dups ? "-Dups" : ""));
         this.spec = spec;
         this.dups = dups;
         envHome = new File(System.getProperty(TestUtils.DEST_DIR));
     }
 
+    @Override
     public void setUp()
-        throws IOException {
+        throws Exception {
+
+        super.setUp();
 
         TestUtils.removeLogFiles("Setup", envHome, false);
         TestUtils.removeFiles("Setup", envHome, FileManager.DEL_SUFFIX);
     }
-    
+
+    @Override
     public void tearDown()
         throws Exception {
 
+        super.tearDown();
+
         try {
-            if (env != null) {
-                env.close();
-            }
-        } catch (Throwable e) {
-            System.out.println("tearDown: " + e);
-        }
-                
-        try {
-            //*
-            TestUtils.removeLogFiles("tearDown", envHome, true);
             TestUtils.removeFiles("tearDown", envHome, FileManager.DEL_SUFFIX);
-            //*/
         } catch (Throwable e) {
             System.out.println("tearDown: " + e);
         }
@@ -311,7 +312,7 @@ public class PhantomRestartTest extends TestCase {
         envConfig.setAllowCreate(true);
         envConfig.setTransactional(true);
         envConfig.setTxnSerializableIsolation(true);
-        
+
         /* Disable the daemons so the don't interfere with stats. */
         envConfig.setConfigParam
             (EnvironmentParams.ENV_RUN_EVICTOR.getName(), "false");
@@ -321,7 +322,7 @@ public class PhantomRestartTest extends TestCase {
             (EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
         envConfig.setConfigParam
             (EnvironmentParams.ENV_RUN_INCOMPRESSOR.getName(), "false");
-       
+
         env = new Environment(envHome, envConfig);
 
         DatabaseConfig dbConfig = new DatabaseConfig();
@@ -392,7 +393,7 @@ public class PhantomRestartTest extends TestCase {
 
         /* Close T1 to allow T2 and T3 to finish. */
         cursor.close();
-        readerTxn.commitNoSync();
+        readerTxn.commit(Durability.COMMIT_NO_SYNC);
         waitForInsert();
         waitForReadOper();
 
@@ -425,12 +426,12 @@ public class PhantomRestartTest extends TestCase {
             } else {
                 status = db.putNoOverwrite(writerTxn, key, data);
             }
-        } catch (DeadlockException e) {
+        } catch (LockConflictException e) {
             writerTxn.abort();
             throw e;
         }
         assertEquals(OperationStatus.SUCCESS, status);
-        writerTxn.commitNoSync();
+        writerTxn.commit(Durability.COMMIT_NO_SYNC);
     }
 
     /**
@@ -439,10 +440,10 @@ public class PhantomRestartTest extends TestCase {
     private void startInsert(final int keyVal, final int dataVal)
         throws DatabaseException, InterruptedException {
 
-        LockStats origStats = env.getLockStats(null);
+        EnvironmentStats origStats = env.getStats(null);
 
         writerThread = new JUnitThread("Writer") {
-            public void testBody() 
+            public void testBody()
                 throws DatabaseException {
                 DatabaseEntry key = new DatabaseEntry();
                 DatabaseEntry data = new DatabaseEntry();
@@ -456,7 +457,7 @@ public class PhantomRestartTest extends TestCase {
                     status = db.putNoOverwrite(writerTxn, key, data);
                 }
                 assertEquals(OperationStatus.SUCCESS, status);
-                writerTxn.commitNoSync();
+                writerTxn.commit(Durability.COMMIT_NO_SYNC);
             }
         };
 
@@ -485,17 +486,17 @@ public class PhantomRestartTest extends TestCase {
     private void startReadOper(final int operKeyParam)
         throws DatabaseException, InterruptedException {
 
-        LockStats origStats = env.getLockStats(null);
+        EnvironmentStats origStats = env.getStats(null);
 
         readerThread = new JUnitThread("Reader") {
-            public void testBody() 
+            public void testBody()
                 throws DatabaseException {
                 Transaction readerTxn = env.beginTransaction(null, null);
                 Cursor cursor = db.openCursor(readerTxn, null);
                 spec.oper.init(PhantomRestartTest.this, cursor);
                 spec.oper.doOper(operKeyParam);
                 cursor.close();
-                readerTxn.commitNoSync();
+                readerTxn.commit(Durability.COMMIT_NO_SYNC);
             }
         };
 
@@ -506,7 +507,7 @@ public class PhantomRestartTest extends TestCase {
     /**
      * Waits for a new locker to block waiting for a lock.
      */
-    private void waitForBlock(LockStats origStats)
+    private void waitForBlock(EnvironmentStats origStats)
         throws DatabaseException, InterruptedException {
 
         long startTime = System.currentTimeMillis();
@@ -520,7 +521,7 @@ public class PhantomRestartTest extends TestCase {
             }
 
             /* Wait for the operation to block. */
-            LockStats stats = env.getLockStats(null);
+            EnvironmentStats stats = env.getStats(null);
             if (stats.getNWaiters() > origStats.getNWaiters()) {
                 break;
             }
