@@ -1,10 +1,8 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002-2006
- *      Sleepycat Software.  All rights reserved.
+ * Copyright (c) 2002-2010 Oracle.  All rights reserved.
  *
- * $Id: TxnFSyncTest.java,v 1.8 2006/01/03 21:56:31 bostic Exp $
  */
 
 package com.sleepycat.je.txn;
@@ -15,7 +13,6 @@ import com.sleepycat.je.Database;
 import com.sleepycat.je.DatabaseConfig;
 import com.sleepycat.je.DatabaseEntry;
 import com.sleepycat.je.DbInternal;
-import com.sleepycat.je.Environment;
 import com.sleepycat.je.EnvironmentConfig;
 import com.sleepycat.je.LockMode;
 import com.sleepycat.je.OperationStatus;
@@ -23,8 +20,8 @@ import com.sleepycat.je.Transaction;
 import com.sleepycat.je.config.EnvironmentParams;
 import com.sleepycat.je.dbi.DbEnvPool;
 import com.sleepycat.je.dbi.EnvironmentImpl;
-import com.sleepycat.je.test.TxnTestCase;
 import com.sleepycat.je.util.TestUtils;
+import com.sleepycat.util.test.TxnTestCase;
 
 /*
  * Make sure that transactions sync to disk. Mimic a crash by failing to
@@ -32,9 +29,9 @@ import com.sleepycat.je.util.TestUtils;
  * properly written and synced data to disk, we'll have unflushed data and
  * we won't find the expected data in the log.
  *
- * Note that this test is run with the TxnTestCase framework and will 
+ * Note that this test is run with the TxnTestCase framework and will
  * be exercised with app-created and autocommit txns.
- */ 
+ */
 
 public class TxnFSyncTest extends TxnTestCase {
 
@@ -46,21 +43,23 @@ public class TxnFSyncTest extends TxnTestCase {
         setupEnvConfig(envConfig);
     }
 
+    static protected Class<?> testClass = TxnFSyncTest.class;
+
     private static void setupEnvConfig(EnvironmentConfig envConfig) {
         envConfig.setTransactional(true);
         envConfig.setConfigParam(
             EnvironmentParams.ENV_RUN_CHECKPOINTER.getName(), "false");
     }
- 
+
     public static Test suite() {
         /* Run these tests with user and autocommit txns. */
-        return txnTestSuite(TxnFSyncTest.class,
+        return txnTestSuite(testClass,
                             envConfig,
-                            new String [] {TxnTestCase.TXN_USER,
-                                           TxnTestCase.TXN_AUTO});
+                            new String[] {TxnTestCase.TXN_USER,
+                                          TxnTestCase.TXN_AUTO});
     }
-    
-    public void testFSyncButNoClose() 
+
+    public void testFSyncButNoClose()
         throws Exception {
 
         try {
@@ -83,26 +82,26 @@ public class TxnFSyncTest extends TxnTestCase {
                              db.putNoOverwrite(txn, key, data));
             }
             txnCommit(txn);
-            
-            /* 
+
+            /*
              * Now throw away this environment WITHOUT flushing the log
              * manager. We do need to release the environment file lock
              * and all file handles so we can recover in this test and
              * run repeated test cases within this one test program.
              */
-            EnvironmentImpl envImpl = DbInternal.envGetEnvironmentImpl(env);
+            EnvironmentImpl envImpl = DbInternal.getEnvironmentImpl(env);
             envImpl.getFileManager().clear(); // release file handles
             envImpl.getFileManager().close(); // release file lock
             env = null;
             DbEnvPool.getInstance().clear();
 
-            /* 
+            /*
              * Open the environment and database again. The database should
              * exist.
              */
             EnvironmentConfig envConfig2 = TestUtils.initEnvConfig();
             setupEnvConfig(envConfig2);
-            env = new Environment(envHome, envConfig2);
+            env = create(envHome, envConfig2);
             dbConfig.setAllowCreate(false);
             db = env.openDatabase(null, "foo", dbConfig);
 
@@ -122,4 +121,3 @@ public class TxnFSyncTest extends TxnTestCase {
         }
     }
 }
-
